#################################################################################
# Title            : PV_pot.R 
# Description      : Computes the solar PV 
# Author(s)        : Sara Moreno (sara.moreno@bsc.es) 
# Date             : January 2025
# Dependencies     : None 
# Inputs           : [temp] - Daily temperature value (K) 
#                    [rad]  - Daily surface solar radiation downwards value
#                             (W/m2) 
# Outputs          : [PV_CF] - Photovoltaic capacity factor (%) 
#################################################################################

#=======================
# Cross rsds(rad) with tas(temp), and obtain CF.
#=======================

PV_pot <- function(temp,rad)
{
  a1 <- 1.1035e-3 
  a2 <- 1.4e-7
  a3 <- -1.725e-8 #coeff in K

  PV <- a1 * rad + a2 * rad^2 + a3 * rad * temp
  return(PV)
}