
library(CSTools)
library(ClimProjDiags)
library(s2dv)
library(qmap)
library(CSIndicators)

both_capacity_factors <- function(obs_tas, obs_rsds, obs_uas, obs_vas, tas, rsds, uas, vas, era5_wind, mask_ori, gwa_spd, agregation, dates_obs, dates_obs_wi, dates_dcpp, dates_dcpp_wi, fyears, IEC_class, model, config, ncores = 1){
  
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/make_eval_train_dexes_mine.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/GetProbs_2.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/add_window_dim.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/Quantile_Mapping_fake.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/WindCapacityFactor_duplicate.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/wind2power_duplicate.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/wind2CF_duplicate.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/read_pc_duplicate.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/CFSLR.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/PV_pot.R')
  
  output <- list()
  obs_tas <- drop(obs_tas)
  obs_rsds <- drop(obs_rsds)
  obs_tas <- s2dv::Reorder(data = obs_tas, order = c('year','time'))
  obs_rsds <- s2dv::Reorder(data = obs_rsds, order = c('year','time'))
  names(dim(obs_tas))[1] <- 'sdate'
  names(dim(obs_rsds))[1] <- 'sdate'
  indices_remove <- apply(dates_obs, 1, function(x) {which(is.na(x) == T)})
  # Remove 29 Feb from observations
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_tas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_tas)['time']) %in% indices_remove[[1]])
  }
  obs_tas <- ClimProjDiags::Subset(x = obs_tas, along = 'time', indices = keep_indices, drop = FALSE)
  obs_tas <- multiApply::Apply(data = obs_tas, target_dims = 'sdate', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'sdate', ncores = config$ncores)$output1
  obs_rsds <- ClimProjDiags::Subset(x = obs_rsds, along = 'time', indices = keep_indices, drop = FALSE)
  obs_rsds <- multiApply::Apply(data = obs_rsds, target_dims = 'sdate', 
                                fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                                output_dims = 'sdate', ncores = config$ncores)$output1
  dates_obs <- ClimProjDiags::Subset(x = dates_obs, along = 'time', indices = keep_indices, drop = FALSE)
  if (model == 'HadGEM3-GC3.1-MM'){
    obs_tas <- obs_tas[,,,1:360]
    obs_rsds <- obs_rsds[,,,1:360]
  } 
  obs_uas <- drop(obs_uas)
  obs_vas <- drop(obs_vas)
  obs_uas <- s2dv::Reorder(data = obs_uas, order = c('year','timewi'))
  obs_vas <- s2dv::Reorder(data = obs_vas, order = c('year','timewi'))
  names(dim(obs_uas))[2] <- 'time'
  names(dim(obs_vas))[2] <- 'time'
  names(dim(dates_obs_wi))[2] <- 'time'
  # Remove 29 Feb
  indices_remove <- apply(dates_obs_wi, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[1]])
  }
  
  obs_uas <- multiApply::Apply(data = obs_uas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  obs_vas <- multiApply::Apply(data = obs_vas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  sfc_wind_obs <- sqrt(obs_uas^2 + obs_vas^2)
  sfc_wind_obs <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'time', indices = keep_indices, drop = FALSE)
  dates_obs_wi <- ClimProjDiags::Subset(x = dates_obs_wi, along = 'time', indices = keep_indices, drop = FALSE)
  era5_wind <- s2dv::MeanDims(data = era5_wind, dims = c('year','month'))
  ratio <- gwa_spd/era5_wind
  ratio_expanded <- array(ratio, dim = dim(sfc_wind_obs))
  sfc_wind_obs <- sfc_wind_obs * ratio_expanded
  names(dim(sfc_wind_obs))[1] <- 'sdate'
  dcpp_tas <- drop(tas)
  dcpp_rsds <- drop(rsds)
  uas <- drop(uas)
  vas <- drop(vas)
  names(dim(uas))[2] <- 'time'
  names(dim(vas))[2] <- 'time'
  names(dim(dates_dcpp_wi))[2] <- 'time'
  sfc_wind_dcpp <- sqrt(uas^2 + uas^2)
  if (model == 'IPSL-CM6A-LR'){
    dates_first_year_au <- dates_dcpp_wi[format(dates_dcpp_wi, "%Y") == config$sdates[2]]
    indices_keep_au <- which(format(dates_first_year_au, "%H") %in% c("00", "06", "12", "18"))
    sfc_wind_dcpp <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'time', indices = indices_keep_au, drop = FALSE)
    dcpp_expanded <- abind::abind(sfc_wind_dcpp, sfc_wind_dcpp[,,,1459, drop=FALSE], along = 4)
    names(dim(dcpp_expanded)) <- names(dim(sfc_wind_dcpp))
    sfc_wind_dcpp <- dcpp_expanded
    dates_dcpp <- ClimProjDiags::Subset(x = dates_dcpp, along = 'time', indices = indices_keep_au, drop = FALSE)
  }
  sdate_dim <- dim(sfc_wind_obs)['sdate']
  cross <- make_eval_train_dexes_mine(eval.method = 'leave-one-out', amt.points = sdate_dim,
                                      amt.points_cor = NULL, k = 1, tail.out = FALSE)
  dates_first_year_au_wi <- dates_dcpp_wi[1,]
  wind_dcpp_ev_corrected <- array(NA, dim = dim(sfc_wind_dcpp)[2])
  wind_dcpp_ev_corrected <- s2dv::InsertDim(data = wind_dcpp_ev_corrected, posdim = 1, lendim = 1, name = 'sdate')
  wind_dcpp_ev_corrected <- s2dv::Reorder(data = wind_dcpp_ev_corrected, order = c('sdate','time')) #,'member'))
  years_len <- length(config$sdates)-1
  wind_dcpp_tr_corrected <- array(NA, dim = dim(sfc_wind_dcpp[1:years_len,]))
  months <- format(dates_dcpp, "%m")
  seasons <- list(
    DJF = c("12", "01", "02"),
    MAM = c("03", "04", "05"),
    JJA = c("06", "07", "08"),
    SON = c("09", "10", "11"),
    Annual = c('01','02','03','04','05','06','07','08','09','10','11','12')
  )
  
  dates_first_year_au <- dates_obs[format(dates_obs, "%Y") == config$sdates[2]]
  seasons_masks <- lapply(seasons, function(seasons_months) {  # MASK FOR RSDS AND TAS
    format(dates_first_year_au, "%m") %in% seasons_months
  })
  # dates_first_year_au_wi <- dates_dcpp_wi[format(dates_dcpp_wi, "%Y") == config$sdates[2]]
  seasons_masks_wi <- lapply(seasons, function(seasons_months) {  # MASK FOR WIND
    format(dates_first_year_au_wi, "%m") %in% seasons_months
  })
  multiplication <- ifelse(mask_ori == 0, 1.39, 1.29)
  if (IEC_class == 'I'){
    cut_in_speed <- 2
    cut_out_speed <- 25
  } else if (IEC_class == 'II'){
    cut_in_speed <- 4
    cut_out_speed <- 25
  } else if (IEC_class == 'III'){
    cut_in_speed <- 3
    cut_out_speed <- 20
  } else if (IEC_class == 'I/II'){
    cut_in_speed <- 4
    cut_out_speed <- 25
  } else if (IEC_class == 'II/III'){
    cut_in_speed <- 3
    cut_out_speed <- 20
  }
  
  output$both_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$both_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$both_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$both_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$both_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$both_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$both_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$both_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$both_obs_SON <- array(NA, dim = c(sdate_dim))
  output$both_obs_Annual <- array(NA, dim = c(sdate_dim))
  output$wind_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$wind_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$wind_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$wind_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$wind_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$wind_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$wind_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$wind_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$wind_obs_SON <- array(NA, dim = c(sdate_dim))
  output$wind_obs_Annual <- array(NA, dim = c(sdate_dim))
  output$solar_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$solar_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$solar_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$solar_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$solar_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$solar_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$solar_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$solar_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$solar_obs_SON <- array(NA, dim = c(sdate_dim))
  output$solar_obs_Annual <- array(NA, dim = c(sdate_dim))
  output$none_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$none_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$none_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$none_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$none_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$none_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$none_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$none_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$none_obs_SON <- array(NA, dim = c(sdate_dim))
  output$none_obs_Annual <- array(NA, dim = c(sdate_dim))
  
  for (t in 1:sdate_dim){
    print(t)
    obs_tas_tr <- ClimProjDiags::Subset(x = obs_tas, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    obs_rsds_tr <- ClimProjDiags::Subset(x = obs_rsds, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    obs_tas_ev <- ClimProjDiags::Subset(x = obs_tas, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    obs_rsds_ev <- ClimProjDiags::Subset(x = obs_rsds, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    dcpp_tas_tr <- ClimProjDiags::Subset(x = dcpp_tas, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    dcpp_rsds_tr <- ClimProjDiags::Subset(x = dcpp_rsds, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    dcpp_tas_ev <- ClimProjDiags::Subset(x = dcpp_tas, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    dcpp_rsds_ev <- ClimProjDiags::Subset(x = dcpp_rsds, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    obs_rsds_tr_wi <- add_window_dim(obs_rsds_tr)
    dcpp_rsds_tr_wi <- add_window_dim(dcpp_rsds_tr)
    dcpp_rsds_ev <- Quantile_Mapping_fake(exp = dcpp_rsds_tr_wi, obs = obs_rsds_tr_wi,
                                          exp_cor = dcpp_rsds_ev,
                                          na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                          window_dim = 'window',
                                          ncores = config$ncores, method = 'QUANT', qstep = 0.01)
    dcpp_rsds_tr <- Quantile_Mapping_fake(exp = dcpp_rsds_tr_wi, obs = obs_rsds_tr_wi,
                                          exp_cor = dcpp_rsds_tr,
                                          na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                          window_dim = 'window',
                                          ncores = config$ncores, method = 'QUANT', qstep = 0.01)
    dcpp_rsds_ev <- s2dv::Reorder(data = dcpp_rsds_ev, order = c('sdate','time'))
    rm(dcpp_rsds_tr_wi,obs_rsds_tr_wi)
    obs_tas_tr_wi <- add_window_dim(obs_tas_tr)
    dcpp_tas_tr_wi <- add_window_dim(dcpp_tas_tr)
    dcpp_tas_ev <- Quantile_Mapping_fake(exp = dcpp_tas_tr_wi, obs = obs_tas_tr_wi,
                                         exp_cor = dcpp_tas_ev,
                                         na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                         window_dim = 'window',
                                         ncores = config$ncores, method = 'QUANT', qstep = 0.01)
    dcpp_tas_tr <- Quantile_Mapping_fake(exp = dcpp_tas_tr_wi, obs = obs_tas_tr_wi,
                                         exp_cor = dcpp_tas_tr,
                                         na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate',
                                         window_dim = 'window',
                                         ncores = config$ncores, method = 'QUANT', qstep = 0.01)
    rm(dcpp_tas_tr_wi,obs_tas_tr_wi)
    dcpp_tas_ev <- s2dv::Reorder(data = dcpp_tas_ev, order = c('sdate','time'))
    wind_obs_tr <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    wind_dcpp_tr <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    wind_dcpp_ev <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    wind_obs_ev <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    for (hour in c("00", "06", "12", "18")){
      indices_hour <- which(format(dates_first_year_au_wi, "%H") == hour)
      wind_dcpp_ev_hour <- ClimProjDiags::Subset(x = wind_dcpp_ev, along = 'time', indices = indices_hour, drop = FALSE)
      wind_dcpp_tr_hour <- ClimProjDiags::Subset(x = wind_dcpp_tr, along = 'time', indices = indices_hour, drop = FALSE)
      wind_obs_tr_hour <- ClimProjDiags::Subset(x = wind_obs_tr, along = 'time', indices = indices_hour, drop = FALSE)
      wind_dcpp_tr_hour_wi <- add_window_dim(wind_dcpp_tr_hour)
      wind_obs_tr_hour_wi <- add_window_dim(wind_obs_tr_hour)
      
      wind_dcpp_tr_hour <- Quantile_Mapping_fake(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                                 exp_cor = wind_dcpp_tr_hour,
                                                 na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                                 window_dim = 'window',
                                                 ncores = config$ncores, method = 'QUANT', qstep = 0.01)
      
      wind_dcpp_ev_hour <- Quantile_Mapping_fake(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                                 exp_cor = wind_dcpp_ev_hour, 
                                                 na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                                 window_dim = 'window',
                                                 ncores = config$ncores, method = 'QUANT', qstep = 0.01)
      wind_dcpp_ev_corrected[,indices_hour] <- wind_dcpp_ev_hour
      wind_dcpp_tr_corrected[,indices_hour] <- wind_dcpp_tr_hour
    }
    multiplication_exp <- array(multiplication, dim = dim(wind_obs_tr))
    wind_obs_tr <- wind_obs_tr * multiplication_exp
    wind_dcpp_tr <- wind_dcpp_tr_corrected * multiplication_exp
    multiplication_exp <- array(multiplication, dim = dim(wind_obs_ev))
    wind_obs_ev <- wind_obs_ev * multiplication_exp
    wind_dcpp_ev_corrected <- wind_dcpp_ev_corrected * multiplication_exp
    
    for (season in names(seasons)) {
      # transformations of arrays to have seasonal data
      seasons_mask <- seasons_masks[[season]]
      seasons_mask_wi <- seasons_masks_wi[[season]]
      
      wind_dcpp_ev_season <- wind_dcpp_ev_corrected[,seasons_mask_wi]
      wind_dcpp_ev_season <- s2dv::InsertDim(data = wind_dcpp_ev_season, posdim = 1, lendim = 1, name = 'sdate')
      names(dim(wind_dcpp_ev_season))[2] <- 'time'
      wind_obs_ev_season <- wind_obs_ev[,seasons_mask_wi]
      wind_obs_ev_season <- s2dv::InsertDim(data = wind_obs_ev_season, posdim = 1, lendim = 1, name = 'sdate')
      names(dim(wind_obs_ev_season))[2] <- 'time'
      dcpp_tas_ev_sea <- dcpp_tas_ev[,seasons_mask]
      dcpp_tas_ev_sea <- s2dv::InsertDim(data = dcpp_tas_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
      names(dim(dcpp_tas_ev_sea))[2] <- 'time'
      obs_tas_ev_sea <- obs_tas_ev[,seasons_mask]
      obs_tas_ev_sea <- s2dv::InsertDim(data = obs_tas_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
      names(dim(obs_tas_ev_sea))[2] <- 'time'
      dcpp_rsds_ev_sea <- dcpp_rsds_ev[,seasons_mask]
      dcpp_rsds_ev_sea <- s2dv::InsertDim(data = dcpp_rsds_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
      names(dim(dcpp_rsds_ev_sea))[2] <- 'time'
      obs_rsds_ev_sea <- obs_rsds_ev[,seasons_mask]
      obs_rsds_ev_sea <- s2dv::InsertDim(data = obs_rsds_ev_sea, posdim = 1, lendim = 1, name = 'sdate')
      names(dim(obs_rsds_ev_sea))[2] <- 'time'
      
      dates_season_sol <- dates_dcpp[,seasons_mask]
      dates_season_wi <- dates_dcpp_wi[,seasons_mask_wi]
      days_sol <- format(dates_season_sol[1, ], "%Y-%m-%d")
      days_wind <- format(dates_season_wi[1, ], "%Y-%m-%d")
      unique_days <- unique(days_wind)
      n_days <- length(unique_days)
      valid_days <- array(FALSE, dim = c(time = n_days))
      valid_days_obs <- array(FALSE, dim = c(time = n_days))
      both_productions <- 0
      wind_productions <- 0
      solar_productions <- 0
      no_production <- 0
      both_productions_obs <- 0
      wind_productions_obs <- 0
      solar_productions_obs <- 0
      no_production_obs <- 0
      
      for (i in seq_len(n_days)) {
        idx_wind <- which(days_wind == unique_days[i])
        idx_sol <- which(days_sol == unique_days[i])
        
        # Extraer valores diarios
        day_vals_wi <- wind_dcpp_ev_season[, idx_wind, drop = FALSE]
        day_vals_obs_wi <- wind_obs_ev_season[, idx_wind, drop = FALSE]
        day_vals_rsds <- dcpp_rsds_ev_sea[, idx_sol, drop = FALSE]
        day_vals_obs_rsds <- obs_rsds_ev_sea[, idx_sol, drop = FALSE]
        day_vals_tas <- dcpp_tas_ev_sea[, idx_sol, drop = FALSE]
        day_vals_obs_tas <- obs_tas_ev_sea[, idx_sol, drop = FALSE]

        cond_wind <- (day_vals_wi > cut_in_speed) & (day_vals_wi < cut_out_speed)
        cond_obs_wind <- (day_vals_obs_wi > cut_in_speed) & (day_vals_obs_wi < cut_out_speed)
        cond_solar <- (day_vals_rsds > 208) & (day_vals_tas < 318)
        cond_obs_solar <- (day_vals_obs_rsds > 208) & (day_vals_obs_tas < 318)
        
        # Wind: contar cuántas horas cumplen la condición en cada miembro
        count_wind <- apply(cond_wind, 1, sum)
        count_obs_wind <- apply(cond_obs_wind, 1, sum)
        
        # Solar: como es diario, no se cuentan horas, solo se comprueba si se cumple
        valid_solar <- as.logical(cond_solar)
        valid_obs_solar <- as.logical(cond_obs_solar)
        
        # Wind: día válido si hay al menos 2 horas en el rango
        valid_wind <- count_wind >= 2
        valid_obs_wind <- count_obs_wind >= 2
        
        # Contar casos por miembro (modelo)
        both_productions <- both_productions + sum(valid_wind & valid_solar)
        wind_productions <- wind_productions + sum(valid_wind & !valid_solar)
        solar_productions <- solar_productions + sum(!valid_wind & valid_solar)
        no_production <- no_production + sum(!valid_wind & !valid_solar)
        
        # Contar casos por miembro (observaciones)
        both_productions_obs <- both_productions_obs + sum(valid_obs_wind & valid_obs_solar)
        wind_productions_obs <- wind_productions_obs + sum(valid_obs_wind & !valid_obs_solar)
        solar_productions_obs <- solar_productions_obs + sum(!valid_obs_wind & valid_obs_solar)
        no_production_obs <- no_production_obs + sum(!valid_obs_wind & !valid_obs_solar)
      }
      
      if (season == 'DJF'){
        output$both_dcpp_DJF[[t]] <- both_productions
        output$wind_dcpp_DJF[[t]] <- wind_productions
        output$solar_dcpp_DJF[[t]] <- solar_productions
        output$none_dcpp_DJF[[t]] <- no_production
        output$both_obs_DJF[[t]] <- both_productions
        output$wind_obs_DJF[[t]] <- wind_productions
        output$solar_obs_DJF[[t]] <- solar_productions
        output$none_obs_DJF[[t]] <- no_production
      } else if (season == 'MAM'){
        output$both_dcpp_MAM[[t]] <- both_productions
        output$wind_dcpp_MAM[[t]] <- wind_productions
        output$solar_dcpp_MAM[[t]] <- solar_productions
        output$none_dcpp_MAM[[t]] <- no_production
        output$both_obs_MAM[[t]] <- both_productions
        output$wind_obs_MAM[[t]] <- wind_productions
        output$solar_obs_MAM[[t]] <- solar_productions
        output$none_obs_MAM[[t]] <- no_production
      } else if (season == 'JJA'){
        output$both_dcpp_JJA[[t]] <- both_productions
        output$wind_dcpp_JJA[[t]] <- wind_productions
        output$solar_dcpp_JJA[[t]] <- solar_productions
        output$none_dcpp_JJA[[t]] <- no_production
        output$both_obs_JJA[[t]] <- both_productions
        output$wind_obs_JJA[[t]] <- wind_productions
        output$solar_obs_JJA[[t]] <- solar_productions
        output$none_obs_JJA[[t]] <- no_production
      } else if (season == 'SON'){
        output$both_dcpp_SON[[t]] <- both_productions
        output$wind_dcpp_SON[[t]] <- wind_productions
        output$solar_dcpp_SON[[t]] <- solar_productions
        output$none_dcpp_SON[[t]] <- no_production
        output$both_obs_SON[[t]] <- both_productions
        output$wind_obs_SON[[t]] <- wind_productions
        output$solar_obs_SON[[t]] <- solar_productions
        output$none_obs_SON[[t]] <- no_production
      } else if (season == 'Annual'){
        output$both_dcpp_Annual[[t]] <- both_productions
        output$wind_dcpp_Annual[[t]] <- wind_productions
        output$solar_dcpp_Annual[[t]] <- solar_productions
        output$none_dcpp_Annual[[t]] <- no_production
        output$both_obs_Annual[[t]] <- both_productions
        output$wind_obs_Annual[[t]] <- wind_productions
        output$solar_obs_Annual[[t]] <- solar_productions
        output$none_obs_Annual[[t]] <- no_production
      }
    }
  }
  return(output)
}
