setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
region <- 'Iberia'
fperiods <- '1'
model <- 'EC-Earth3-i4'
members <- c('r1i4p1f1','r2i4p1f1','r3i4p1f1','r4i4p1f1','r5i4p1f1','r6i4p1f1')
fyears_string <- 'fyear1'
energy <- 'solar'
IEC_class <- 'I'
all_metrics <- list()
results_multimodel <- list()
obs_seasons <- list()

for (season in config$seasons) {
  cat("Processing season:", season, "\n")
  for (i in seq_along(members)) {
    member <- members[i]
    if (energy == 'solar'){
      data <- readRDS(file = paste0(config$capacity_factors_models, 'solar_capacity_factor_',
                                    model, '_', fyears_string, '_', region, '_', member, '_first.rds'))
      pv_dcpp <- drop(data[[paste0("pv_dcpp_", season)]])
      pv_dcpp_probs <- drop(data[[paste0("pv_dcpp_probs_", season)]])
      n_effective_dcpp <- drop(data[[paste0("n_effective_dcpp_", season)]])
    } else if (energy == 'wind'){
      data <- readRDS(file = paste0(config$capacity_factors_models, 'wind_capacity_factor_', 
                                    model,'_',fyears_string,'_', member,'_', region, '_',
                                    'IEC_class_',IEC_class,'.rds'))
      pv_dcpp <- drop(data[[paste0("cf_dcpp_", season)]])
      pv_dcpp_probs <- drop(data[[paste0("cf_probs_", season)]])
      n_effective_dcpp <- drop(data[[paste0("subdays_dcpp_", season)]])
    }

    
    if (i == 1) {
      pv_dcpp_mul <- array(NA, dim = c(dim(pv_dcpp), member = length(members)))
      pv_dcpp_probs_mul <- array(NA, dim = c(dim(pv_dcpp_probs), member = length(members)))
      n_effective_dcpp_mul <- array(NA, dim = c(dim(n_effective_dcpp), member = length(members)))
      if (energy == 'solar'){
        pv_obs <- drop(data[[paste0("pv_obs_", season)]])
        n_effective_obs <- drop(data[[paste0("n_effective_obs_", season)]])
      } else if (energy == 'wind'){
        pv_obs <- drop(data[[paste0("cf_obs_", season)]])
        n_effective_obs <- drop(data[[paste0("subdays_obs_", season)]])
      }
      sdate_dim <- dim(pv_obs)["sdate"]
      cross <- CSTools:::.make.eval.train.dexes('leave-one-out', sdate_dim, NULL)
      pv_obs_probs <- array(data = NA, dim = dim(pv_dcpp_probs))
      for (t in 1:sdate_dim) {
        obs_tr <- ClimProjDiags::Subset(x = pv_obs, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
        obs_ev <- ClimProjDiags::Subset(x = pv_obs, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
        lims_ano_obs_tr <- Apply(obs_tr, target_dims = 'sdate',
                                 fun = function(x) quantile(as.vector(x), c(1/3, 2/3), na.rm = TRUE),
                                 output_dims = 'bin', ncores = config$ncores)$output1
        
        obs_probs_ev <- GetProbs_2(obs_ev, time_dim = 'sdate',
                                   prob_thresholds = NULL,
                                   bin_dim_abs = 'bin',
                                   indices_for_quantiles = NULL,
                                   memb_dim = NULL,
                                   abs_thresholds = lims_ano_obs_tr,
                                   ncores = config$ncores)
        obs_probs_ev <- drop(obs_probs_ev)
        pv_obs_probs[,t,,] <- Reorder(data = obs_probs_ev, order = c('bin','lon','lat'))
      }
    }
    pv_dcpp_mul[,,,i] <- pv_dcpp
    pv_dcpp_probs_mul[,,,,i] <- pv_dcpp_probs
    n_effective_dcpp_mul[,,,i] <- n_effective_dcpp
  }
  metrics_cf <- compute_metrics(dcpp = pv_dcpp_mul, obs = pv_obs, model = model,
                                dcpp_probs = pv_dcpp_probs_mul, obs_probs = pv_obs_probs, time_dim = 'sdate')
  metrics_n_effe <- CorrEno(exp = n_effective_dcpp_mul, obs = n_effective_obs, time_dim = 'sdate', member_dim = 'member', 
                            method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
  all_metrics[[season]] <- list(cf = metrics_cf,
                                n_effective = metrics_n_effe)
  pv_multimodel <- MeanDims(data = pv_dcpp_mul, dims = c('member','sdate'))
  n_efe_multimodel <- MeanDims(data = n_effective_dcpp_mul, dims = c('member','sdate'))
  results_multimodel[[season]] <- list(pv_multimodel = pv_multimodel,
                                   n_efe_multimodel = n_efe_multimodel)
  obs_seasons[[season]] <- list(pv_obs = pv_obs,
                        n_effective_obs = n_effective_obs)
}
if (energy == 'solar'){
  saveRDS(object = list(metrics = all_metrics,
                        results_multimodel = results_multimodel,
                        obs_seasons = obs_seasons,
                        lon = data$lon,
                        lat = data$lat,
                        config = config),
          file = paste0(config$metrics_capacity, 'metrics_',energy,'_CF_', model, '_', fyears_string, '_', region,'.rds'))
  
} else if (energy == 'wind'){
  saveRDS(object = list(metrics = all_metrics,
                        results_multimodel = results_multimodel,
                        obs_seasons = obs_seasons,
                        lon = data$lon,
                        lat = data$lat,
                        config = config),
          file = paste0(config$metrics_capacity, 'metrics_',energy,'_CF_', model, '_', fyears_string, '_', region, '_IEC_class_',IEC_class,'.rds'))
  
}
