setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
source('/esarchive/scratch/smoreno/gitlab/capacity_factors/simpl_wind_capacity_factors.R')

models <- config$models
region <- 'Iberia'
fperiods <- '1'
model <- 'EC-Earth3-i4'
agregation <- 'seasons'
retrieve <- FALSE
IEC_class <- 'I'
members <- 'r1i4p1f1'
grid <- config$grid_25
fy <- 1

retrieve <- TRUE
config$years <- 1990:1992
config$sdates <- 1989:1991
config$regions = list('Iberia' = list('lon_min' = 0, 'lon_max' = 0.3, 'lat_min' = 41, 'lat_max' = 41.3))
if (identical(models,'all')){
  models <- c(config$models,config$multimodels)
} else {
  models <- unlist(strsplit(x = models, split = ','))
}
if (identical(fperiods,'all')){
  fperiods <- config$fperiods
} else {
  aux <- unlist(strsplit(x = fperiods, split = ','))
  fperiods <- list()
  for (i in 1:length(aux)){
    auxi <- unlist(strsplit(x = aux[i], split = '-'))
    fperiods[[i]] <- seq(from = auxi[1], to = auxi[length(auxi)], by = 1)
  }
}

  for (fyears in fperiods){
    time_starting_exec <- proc.time()
    
    if (length(fyears) == 1){fyears_string <- paste0('fyear',fyears[1])} else {fyears_string <- paste0('fyears',fyears[1],'-',fyears[length(fyears)])}
    sdates <- config$sdates[1] : (config$years[length(config$years)] - as.integer(fyears[length(fyears)]))
    years <- (sdates[1] + as.integer(fyears[1])) : config$years[length(config$years)]
    # indices_for_clim <- seq(from = (config$reference_period[1]-years[1]+1), length.out = length(config$reference_period)-length(fyears)+1)

    print(paste0('Starting ',model,' - ',fyears_string))
    
    ################################################################################################
    #################################### Loading the data ##########################################
    ################################################################################################
    
    print('Loading the data...')
  
    time_dim <- 'sdate'
    path_era5_solar <- '/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'
    path_era5_wind <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
    path_mask <- '/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/'
    era5_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'  
    gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/GWA_$var$m_t170grid.nc"
    
    
    #### TAS ####
    obs_tas <- startR::Start(dat = path_era5_solar, var = 'tas',
                             lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                             lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                             year = paste0(config$years), month = 'all', time = 'all', time_across = 'month', 
                             merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                             synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                             return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                             lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                             num_procs = 1, retrieve = retrieve)
    
    ##### RSDS #####
    obs_rsds <- startR::Start(dat = path_era5_solar, var = 'rsds',
                              lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                              lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                              year = paste0(config$years), month = 'all', time = 'all',
                              time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                              synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                              return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                              lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                              num_procs = 1, retrieve = retrieve)
    
    dates_obs <- attr(obs_rsds,"Variables")$common$time
    
    total_hours <- 8928
    desired_hours <- c(0, 6, 12, 18)
    selected_indices <- NULL
    for (hour in desired_hours) {
          selected_indices <- c(selected_indices, seq(hour + 1, total_hours, by = 24))
    }
      obs_uas <- startR::Start(dat = path_era5_wind, var = 'uas',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', timewi = selected_indices,
                           timewi_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), timewi = c('time','timewi')),
                           return_vars = list(lat = NULL, lon = NULL, timewi = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = retrieve)
      
      obs_vas <- startR::Start(dat = path_era5_wind, var = 'vas',
                               lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                               lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                               year = paste0(config$years), month = 'all', timewi = selected_indices,
                               timewi_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                               synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), timewi = c('time','timewi')),
                               return_vars = list(lat = NULL, lon = NULL, timewi = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                               lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                               num_procs = 1, retrieve = retrieve)
    
      dates_obs_wi <- attr(obs_vas,"Variables")$common$timewi
      lon <- attr(obs_vas,'Variables')$common[['lon']]
      lat <- attr(obs_vas,'Variables')$common[['lat']]
      
      era5_wind <- startR::Start(dat = era5_path, var = 'sfcWind',
                                 lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                 lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                 year = paste0(config$years), month = 'all', timewi = 'all',
                                 synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude'), timewi = c('time','timewi')),
                                 return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                                 lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                 transform = CDORemapper,
                                 transform_extra_cells = 2,
                                 transform_params = list(grid = grid,
                                                         method = 'conservative'),
                                 transform_vars = c('lat', 'lon'),
                                 num_procs = 1, retrieve = retrieve)

      gwa <- startR::Start(dat = gwa100_file, var = 'WSPD100',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           transform = CDORemapper,
                           transform_extra_cells = 2,
                           transform_params = list(grid = grid,
                                                   method = 'conservative'),
                           transform_vars = c('lat', 'lon'),
                           num_procs = 1, retrieve = retrieve)
      
      mask_ori <- startR::Start(dat = paste0(path_mask,'$var$.nc'), var = 'sftof',
                                lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                                lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                transform = CDORemapper,
                                transform_extra_cells = 2,
                                transform_params = list(grid = grid,
                                                        method = 'conservative'),
                                transform_vars = c('lat', 'lon'),
                                num_procs = 1, retrieve = retrieve)
      
      
    if (identical(model,'MRI-ESM2-0')) {
      indices_fyears <- fyears[-which(fyears > 4)]
    } else if (identical(model,'MRI-ESM2-0') & any(fyears > 5)){
      indices_fyears <- fyears[-which(fyears > 5)]
    } else {indices_fyears <- fyears}
      indices_fyears <- 1
      # tas <- startR_temporal_files(dataset = model, variables = 'tas', fyears = fy, config = config,retrieve = retrieve, members = member, region = region)
      # rsds <- startR_temporal_files(dataset = model, variables = 'rsds', fyears = fy, config = config, retrieve = retrieve, members = member, region = region)
      # dcpp_wind <- startR_wind_temporal_files(model = model,fy = fy, config = config, member = member, machine_index = machine_index,
      #                                 retrieve = retrieve, region = region)
      
      
      #########################
      tas <- startR_calls_daily(dataset = model, variables = 'tas', fyears = fy, config = config, lon_min = config$regions[[region]]$lon_min, lon_max = config$regions[[region]]$lon_max,
                                lat_min = config$regions[[region]]$lat_min, lat_max = config$regions[[region]]$lat_max, retrieve = retrieve, members = members, interpolation = TRUE)
      rsds <- startR_calls_daily(dataset = model, variables = 'rsds', fyears = fy, config = config, lon_min = config$regions[[region]]$lon_min, lon_max = config$regions[[region]]$lon_max,
                                 lat_min = config$regions[[region]]$lat_min, lat_max = config$regions[[region]]$lat_max, retrieve = retrieve, members = members, interpolation = TRUE)
      dcpp <- startR_calls_subdaily_wind(model = model,fy = fy, config = config, lon_min = config$regions[[region]]$lon_min, lon_max = config$regions[[region]]$lon_max, 
                                         lat_min = config$regions[[region]]$lat_min, lat_max = config$regions[[region]]$lat_max, member = members, machine_index = machine_index,
                                         retrieve = retrieve)
      #########################
      }
          output_dims <- list(both_dcpp_DJF= c('sdate'),
                              both_dcpp_MAM= c('sdate'),
                              both_dcpp_JJA= c('sdate'),
                              both_dcpp_SON= c('sdate'),
                              both_dcpp_Annual= c('sdate'),
                              both_obs_DJF= c('sdate'),
                              both_obs_MAM= c('sdate'),
                              both_obs_JJA= c('sdate'),
                              both_obs_SON= c('sdate'),
                              both_obs_Annual= c('sdate'),
                              wind_dcpp_DJF= c('sdate'),
                              wind_dcpp_MAM= c('sdate'),
                              wind_dcpp_JJA= c('sdate'),
                              wind_dcpp_SON= c('sdate'),
                              wind_dcpp_Annual= c('sdate'),
                              wind_obs_DJF= c('sdate'),
                              wind_obs_MAM= c('sdate'),
                              wind_obs_JJA= c('sdate'),
                              wind_obs_SON= c('sdate'),
                              wind_obs_Annual= c('sdate'),
                              solar_dcpp_DJF= c('sdate'),
                              solar_dcpp_MAM= c('sdate'),
                              solar_dcpp_JJA= c('sdate'),
                              solar_dcpp_SON= c('sdate'),
                              solar_dcpp_Annual= c('sdate'),
                              solar_obs_DJF= c('sdate'),
                              solar_obs_MAM= c('sdate'),
                              solar_obs_JJA= c('sdate'),
                              solar_obs_SON= c('sdate'),
                              solar_obs_Annual= c('sdate'),
                              none_dcpp_DJF= c('sdate'),
                              none_dcpp_MAM= c('sdate'),
                              none_dcpp_JJA= c('sdate'),
                              none_dcpp_SON= c('sdate'),
                              none_dcpp_Annual= c('sdate'),
                              none_obs_DJF = c('sdate'),
                              none_obs_MAM = c('sdate'),
                              none_obs_JJA = c('sdate'),
                              none_obs_SON = c('sdate'),
                              none_obs_Annual = c('sdate'))

          target_dims <-  list(obs_tas = c('year','time'),
                               obs_rsds = c('year','time'),
                               obs_uas = c('year', 'timewi'),
                               obs_vas = c('year', 'timewi'),
                               tas = c('sdate','time','member'),
                               rsds = c('sdate','time','member'),
                               dcpp_uas = c('sdate','timewi','member'),
                               dcpp_vas = c('sdate','timewi','member'),
                               era5_wind = c('year','month'),
                               mask_ori = c('var'),
                               gwa_spd = c('var'))
          
          if (isTRUE(retrieve)) {

            output <- multiApply::Apply(data = list(obs_tas = obs_tas,
                                                    obs_rsds = obs_rsds,
                                                    obs_uas = obs_uas,
                                                    obs_vas = obs_vas,
                                                    tas = tas$data,
                                                    rsds = rsds$data,
                                                    dcpp_uas = dcpp$uas,
                                                    dcpp_vas = dcpp$vas,
                                                    era5_wind = era5_wind,
                                                    mask_ori = mask_ori, 
                                                    gwa_spd = gwa 
                                                    ),
                                        agregation = agregation,
                                        dates_obs = dates_obs,
                                        dates_obs_wi = dates_obs_wi,
                                        dates_dcpp = rsds$dates,
                                        dates_dcpp_wi = dcpp$dates,
                                        fyears = 1,
                                        IEC_class = 'I',
                                        model = model,
                                        output_dims = output_dims,
                                        target_dims = target_dims,
                                        fun = both_capacity_factors,
                                        config = config,
                                        ncores = config$ncores)
            
          } else if (isFALSE(retrieve)) {
            
            step <- startR::Step(fun = simpl_wind_capacity_factors,
                                 target_dims = target_dims, 
                                 output_dims = output_dims)
            
            wf <- startR::AddStep(inputs = list(obs_tas = obs_tas,
                                                obs_rsds = obs_rsds,
                                                obs_uas = obs_uas, 
                                                obs_vas = obs_vas, 
                                                tas = tas$data,
                                                rsds = rsds$data,
                                                dcpp_uas = dcpp$uas, 
                                                dcpp_vas = dcpp$vas, 
                                                era5_wind = era5_wind, 
                                                mask_ori = mask_ori, 
                                                gwa_spd = gwa),
                                  agregation = agregation,
                                  dates_obs = dates_obs,
                                  dates_dcpp = dcpp$dates,
                                  fyears = 1,
                                  IEC_class = 'I',
                                  model = model,
                                  config = config,
                                  ncores = config$ncores,
                                  step_fun = step)
            
            # if (dataset %in% config$models){
            #   chunks <- list(member = as.numeric(attr(data$data,'Dimensions')['member']),
            #                  lat = 10)
            #   job_wallclock <- '01:00:00'
            # } else {
              chunks <- list(lat = 10, lon = 10) #, member = 2)
              job_wallclock <- '48:00:00'
            # }
            

            output <- Compute(workflow = wf[[1]],
                              chunks = chunks,
                              threads_load = 1,
                              threads_compute = config$ncores,
                              cluster = list(queue_host = 'nord4',
                                             r_module = "R/4.1.2-foss-2019b",
                                             temp_dir = '/gpfs/scratch/bsc32/bsc032693/startR_hpc/',
                                             CDO_module = "CDO/1.9.8-foss-2019b",
                                             ecflow_module = 'ecFlow',
                                             cores_per_job = 8,
                                             job_wallclock = job_wallclock,
                                             max_jobs = config$max_jobs,
                                             special_setup = "nord4",
                                             extra_queue_params = list('#SBATCH -A bsc32', '#SBATCH -q bsc_es'),
                                             bidirectional = FALSE,
                                             polling_period = 10),
                              ecflow_suite_dir = '/home/Earth/smoreno/Desktop/startR_local/',
                              wait = TRUE)
            
            
          } else {stop('retrieve must be TRUE or FALSE')}
     
          saveRDS(object = list(n_effective_dcpp_DJF = output$n_effective_dcpp_DJF,
                                 n_effective_dcpp_MAM = output$n_effective_dcpp_MAM,
                                 n_effective_dcpp_JJA = output$n_effective_dcpp_JJA,
                                 n_effective_dcpp_SON = output$n_effective_dcpp_SON,
                                 n_effective_dcpp_Annual = output$n_effective_dcpp_Annual,
                                 n_effective_obs_DJF = output$n_effective_obs_DJF,
                                 n_effective_obs_MAM = output$n_effective_obs_MAM,
                                 n_effective_obs_JJA = output$n_effective_obs_JJA,
                                 n_effective_obs_SON = output$n_effective_obs_SON,
                                 n_effective_obs_Annual = output$n_effective_obs_Annual,
                                 lon = lon,
                                 lat = lat,
                                 config = config,
                                 dates = dcpp$dates),
                  file = paste0(config$capacity_factors_models,'effective_days_solar_and_wind_',model,'_',fyears_string,'_',member,'_',region,'_IEC_class_',IEC_class,'.rds'))
          # file = paste0(config$capacity_factors_models,'wind_capacity_factor_test.rds'))
    
    print('FINISHED :D')
  
