setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
models <- config$models
region <- 'Iberia'
fperiods <- '1-3'
retrieve <- TRUE
config$years <- 1990:1991 

grid <- config$grid_25
config$regions = list('Iberia' = list('lon_min' = 0, 'lon_max' = 0.2, 'lat_min' = 41, 'lat_max' = 41.2))

if (identical(models,'all')){
  models <- c(config$models,config$multimodels)
} else {
  models <- unlist(strsplit(x = models, split = ','))
}
if (identical(fperiods,'all')){
  fperiods <- config$fperiods
} else {
  aux <- unlist(strsplit(x = fperiods, split = ','))
  fperiods <- list()
  for (i in 1:length(aux)){
    auxi <- unlist(strsplit(x = aux[i], split = '-'))
    fperiods[[i]] <- seq(from = auxi[1], to = auxi[length(auxi)], by = 1)
  }
}

  for (fyears in fperiods){
    
    time_starting_exec <- proc.time()
    
    if (length(fyears) == 1){fyears_string <- paste0('fyear',fyears[1])} else {fyears_string <- paste0('fyears',fyears[1],'-',fyears[length(fyears)])}
    sdates <- config$sdates[1] : (config$years[length(config$years)] - as.integer(fyears[length(fyears)]))
    years <- (sdates[1] + as.integer(fyears[1])) : config$years[length(config$years)]
    # indices_for_clim <- seq(from = (config$reference_period[1]-years[1]+1), length.out = length(config$reference_period)-length(fyears)+1)
    
    print('Loading the data...')
    
    time_dim <- 'sdate'
    sys_info <- Sys.info()['nodename']
    machine_index <- substr(sys_info, 1, 2)
    path <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
    path_mask <- '/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/'
    path_obs_100 <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
    era5_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'  
    gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/GWA_$var$m_t170grid.nc"

    total_hours <- 8928
    desired_hours <- c(0, 6, 12, 18)
    selected_indices <- NULL
    for (hour in desired_hours) {
          selected_indices <- c(selected_indices, seq(hour + 1, total_hours, by = 24))
    }
      obs_uas <- startR::Start(dat = path, var = 'uas',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', time = selected_indices,
                           time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = retrieve)

      obs_vas <- startR::Start(dat = path, var = 'vas',
                               lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                               lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                               year = paste0(config$years), month = 'all', time = selected_indices,
                               time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                               synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                               return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                               lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                               num_procs = 1, retrieve = retrieve)
    
      dates_obs <- attr(obs_vas,"Variables")$common$time
      lon <- attr(obs_vas,'Variables')$common[['lon']]
      lat <- attr(obs_vas,'Variables')$common[['lat']]
      
      era5_wind <- startR::Start(dat = era5_path, var = 'sfcWind',
                                 lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                 lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                 year = paste0(config$years), month = 'all', time = 'all',
                                 synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                 return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                                 lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                 transform = CDORemapper,
                                 transform_extra_cells = 2,
                                 transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/uas-r1440x721cds/uas_199001.nc'),
                                                         method = 'bil'),
                                 transform_vars = c('lat', 'lon'),
                                 num_procs = 1, retrieve = retrieve)

      gwa <- startR::Start(dat = gwa100_file, var = 'WSPD100',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           transform = CDORemapper,
                           transform_extra_cells = 2,
                           transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/uas-r1440x721cds/uas_199001.nc'),
                                                   method = 'bil'),
                           transform_vars = c('lat', 'lon'),
                           num_procs = 1, retrieve = retrieve)
      
      mask_ori <- startR::Start(dat = paste0(path_mask,'$var$.nc'), var = 'sftof',
                                lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                                lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                transform = CDORemapper,
                                transform_extra_cells = 2,
                                transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/uas-r1440x721cds/uas_199001.nc'),
                                                        method = 'bil'),
                                transform_vars = c('lat', 'lon'),
                                num_procs = 1, retrieve = retrieve)
      
      obs_uas_100 <-  startR::Start(dat = path_obs_100, var = 'u100m',
                                    lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                    lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                    year = paste0(config$years), month = 'all', time = selected_indices,
                                    time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                                    synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                    return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                                    lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                    num_procs = 1, retrieve = retrieve)
      
      obs_vas_100 <- startR::Start(dat = path_obs_100, var = 'v100m',  
                                   lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                   lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                   year = paste0(config$years), month = 'all', time = selected_indices,
                                   time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                                   synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                   return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                                   lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                   num_procs = 1, retrieve = retrieve)
      output_dims <- list(
        wind_100_DJF = character(0),
        wind_100_MAM = character(0),
        wind_100_JJA = character(0),
        wind_100_SON = character(0),
        wind_100_Annual = character(0),
        sfc_wind_DJF = character(0),
        sfc_wind_MAM = character(0),
        sfc_wind_JJA = character(0),
        sfc_wind_SON = character(0),
        sfc_wind_Annual = character(0)
      )

          target_dims <-  list(obs_uas = c('year', 'time'),
                               obs_vas = c('year', 'time'),
                               obs_uas_100 = c('year', 'time'),
                               obs_vas_100 = c('year', 'time'),
                               era5_wind = c('year','month'),
                               mask_ori = c('var'),
                               gwa_spd = c('var'))
          
          if (isTRUE(retrieve)) {
            output <- multiApply::Apply(data = list(obs_uas = obs_uas, 
                                                    obs_vas = obs_vas, 
                                                    obs_uas_100 = obs_uas_100, 
                                                    obs_vas_100 = obs_vas_100, 
                                                    mask_ori = mask_ori #InsertDim(data = mask_ori[,,1:2,1:2], posdim = 1, lendim = 1, name = 'var'),
                                                    ),
                                        dates_obs = dates_obs,
                                        output_dims = output_dims, 
                                        target_dims = target_dims, 
                                        fun = climatologies_era5, 
                                        config = config, 
                                        ncores = config$ncores)

          } else if (isFALSE(retrieve)) {
            
            step <- startR::Step(fun = climatologies_era5,
                                 target_dims = target_dims, 
                                 output_dims = output_dims)
            
            wf <- startR::AddStep(inputs = list(obs_uas = obs_uas, 
                                                obs_vas = obs_vas,
                                                obs_uas_100 = obs_uas_100, 
                                                obs_vas_100 = obs_vas_100, 
                                                mask_ori = mask_ori,),
                                  agregation = agregation,
                                  dates_obs = dates_obs,
                                  config = config,
                                  ncores = config$ncores,
                                  step_fun = step)

              chunks <- list(lat = length(lat), lon = 23) 
              job_wallclock <- '24:00:00'
              config$ncores <- 4
              
            output <- Compute(workflow = wf[[1]],
                              chunks = chunks,
                              threads_load = 1,
                              threads_compute = config$ncores,
                              cluster = list(queue_host = 'nord4',
                                             r_module = "R/4.1.2-foss-2019b",
                                             temp_dir = '/gpfs/scratch/bsc32/bsc032693/startR_hpc/',
                                             CDO_module = "CDO/1.9.8-foss-2019b",
                                             ecflow_module = 'ecFlow',
                                             cores_per_job = 16,
                                             job_wallclock = job_wallclock,
                                             max_jobs = 300,
                                             special_setup = "nord4",
                                             extra_queue_params = list('#SBATCH -A bsc32', '#SBATCH -q bsc_es'),
                                             bidirectional = FALSE,
                                             polling_period = 10),
                              ecflow_suite_dir = '/home/Earth/smoreno/Desktop/startR_local/',
                              wait = TRUE)
            
            
          } else {stop('retrieve must be TRUE or FALSE')}
          
          
          saveRDS(object = list(wind_100_DJF = output$wind_100_DJF,
                                wind_100_MAM = output$wind_100_MAM,
                                wind_100_JJA = output$wind_100_JJA,
                                wind_100_SON = output$wind_100_SON,
                                wind_100_Annual = output$wind_100_Annual,
                                sfc_wind_DJF = character(0),
                                sfc_wind_MAM = character(0),
                                sfc_wind_JJA = character(0),
                                sfc_wind_SON = character(0),
                                sfc_wind_Annual = character(0),
                                lon = lon,
                                lat = lat,
                                config = config,
                                dates = dcpp$dates),
                  file = paste0(config$capacity_factors_models,'wind_capacity_factor_',model,'_fyear_',fyear,'_',member,'_',region,'_IEC_class_',IEC_class,'.rds'))
          # file = paste0(config$capacity_factors_models,'wind_capacity_factor_test.rds'))
          
          
      }
  

    print(paste0('Finished in ',round(x = (proc.time() - time_starting_exec)[3]/60, digits = 0),' min.'))
  
