setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
source('startR_calls_daily.R')
method <- 'first'
agregation <- 'seasons'
region <- 'Iberia'
grid <- config$grid_25

retrieve <- FALSE
fperiods <- '1'
reference <- 'ERA5'
model <- 'EC-Earth3-i4'
members <- 'r5i4p1f1'
# config$sdates <- 1970:1975
# config$years <- 1971:1976
# config$regions = list('Iberia' = list('lon_min' =1, 'lon_max' = 1.3, 'lat_min' = 40, 'lat_max' = 40.3))

if (identical(fperiods,'all')){
  fperiods <- config$fperiods
} else {
  aux <- unlist(strsplit(x = fperiods, split = ','))
  fperiods <- list()
  for (i in 1:length(aux)){
    auxi <- unlist(strsplit(x = aux[i], split = '-'))
    fperiods[[i]] <- seq(from = auxi[1], to = auxi[length(auxi)], by = 1)
  }
}

fyears <- fperiods
time_starting_exec <- proc.time()
if (length(fyears) == 1){fyears_string <- paste0('fyear',fyears[1])} else {fyears_string <- paste0('fyears',fyears[1],'-',fyears[length(fyears)])}
sdates <- config$sdates[1] : (config$years[length(config$years)] - as.integer(fyears[length(fyears)]))
years <- (sdates[1] + as.integer(fyears[1])) : config$years[length(config$years)]
# indices_for_clim <- seq(from = (config$reference_period[1]-years[1]+1), length.out = length(config$reference_period)-length(fyears)+1)
print(paste0('Starting ',model,' - ',fyears_string, ' - ', members))

################################################################################################
#################################### Loading the data ##########################################
################################################################################################

print('Loading the data...')
time_dim <- 'sdate'
sys_info <- Sys.info()['nodename']
machine_index <- substr(sys_info, 1, 2)
if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
  path <- '/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5land/daily_mean/$var$_f1h/$var$_$year$$month$.nc'
} else {
  if (reference == 'ERA5land'){
    path <- '/esarchive/recon/ecmwf/era5land/daily_mean/$var$_f1h/$var$_$year$$month$.nc'
  } else if (reference == 'ERA5'){
    path <- '/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'
  }
}

#### TAS ####
obs_tas <- startR::Start(dat = path, var = 'tas',
                         lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                         lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                         year = paste0(config$years), month = 'all', time = 'all', time_across = 'month', 
                         merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                         lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                         num_procs = 1, retrieve = retrieve)

##### RSDS #####
obs_rsds <- startR::Start(dat = path, var = 'rsds',
                          lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                          lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                          year = paste0(config$years), month = 'all', time = 'all',
                          time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)

dates_obs <- attr(obs_rsds,'Variables')$common$time
lon <- attr(obs_rsds,'Variables')$common[['lon']]
lat <- attr(obs_rsds,'Variables')$common[['lat']]

if (identical(model,'MRI-ESM2-0')) { #& grepl(pattern = '_djf', x = variable) & any(fyears > 4)
  indices_fyears <- fyears[-which(fyears > 4)]
} else if (identical(model,'MRI-ESM2-0') & any(fyears > 5)){
  indices_fyears <- fyears[-which(fyears > 5)]
} else {indices_fyears <- fyears}
indices_fyears <- 1
fy <- indices_fyears
if (length(fyears) == 1){fyears_string <- paste0('fyear',fyears[1])} else {fyears_string <- paste0('fyears',fyears[1],'-',fyears[length(fyears)])}
sdates <- config$sdates[1] : (config$years[length(config$years)] - as.integer(fyears[length(fyears)]))
years <- (sdates[1] + as.integer(fyears[1])) : config$years[length(config$years)]
tas <- startR_calls_daily(dataset = model, variables = 'tas', fyears = fy, config = config, lon_min = config$regions[[region]]$lon_min, lon_max = config$regions[[region]]$lon_max,
                          lat_min = config$regions[[region]]$lat_min, lat_max = config$regions[[region]]$lat_max, retrieve = retrieve, members = members, interpolation = TRUE)
rsds <- startR_calls_daily(dataset = model, variables = 'rsds', fyears = fy, config = config, lon_min = config$regions[[region]]$lon_min, lon_max = config$regions[[region]]$lon_max,
                           lat_min = config$regions[[region]]$lat_min, lat_max = config$regions[[region]]$lat_max, retrieve = retrieve, members = members, interpolation = TRUE)
dates_dcpp <- rsds$dates

output_dims <- list(pv_dcpp_DJF = 'sdate',
                    pv_dcpp_MAM = 'sdate',
                    pv_dcpp_JJA = 'sdate',
                    pv_dcpp_SON = 'sdate',
                    pv_dcpp_Annual = 'sdate',
                    pv_obs_DJF = 'sdate',
                    pv_obs_MAM = 'sdate',
                    pv_obs_JJA = 'sdate',
                    pv_obs_SON = 'sdate',
                    pv_obs_Annual = 'sdate',
                    pv_dcpp_probs_DJF = c('bin','sdate'),
                    pv_dcpp_probs_MAM = c('bin','sdate'),
                    pv_dcpp_probs_JJA = c('bin','sdate'),
                    pv_dcpp_probs_SON = c('bin','sdate'),
                    pv_dcpp_probs_Annual = c('bin','sdate'),
                    n_effective_dcpp_DJF = 'sdate',
                    n_effective_dcpp_MAM = 'sdate',
                    n_effective_dcpp_JJA = 'sdate',
                    n_effective_dcpp_SON = 'sdate',
                    n_effective_dcpp_Annual = 'sdate',
                    n_effective_obs_DJF = 'sdate',
                    n_effective_obs_MAM = 'sdate',
                    n_effective_obs_JJA = 'sdate',
                    n_effective_obs_SON = 'sdate',
                    n_effective_obs_Annual = 'sdate')

target_dims <-  list(obs_tas = c('year', 'time'),
                     obs_rsds = c('year', 'time'),
                     tas = c('sdate','time','member','exp','initmonth'),
                     rsds = c('sdate','time','member','exp','initmonth'))


if (isTRUE(retrieve)) {
  output <- multiApply::Apply(data = list(obs_tas = obs_tas, 
                                          obs_rsds = obs_rsds, 
                                          tas = tas$data, 
                                          rsds = rsds$data
                                          
  ),
  agregation = agregation,
  dates_obs = dates_obs,
  dates_dcpp = rsds$dates,
  fyears = 1,
  model = model,
  method = 'first',
  output_dims = output_dims, 
  target_dims = target_dims, 
  fun = solar_capacity_factors, 
  config = config, 
  ncores = 1)
  
} else if (isFALSE(retrieve)) {
  
  step <- Step(fun = solar_capacity_factors,
               target_dims = target_dims, 
               output_dims = output_dims)
  
  wf <- AddStep(inputs = list(obs_tas = obs_tas, 
                              obs_rsds = obs_rsds, 
                              tas = tas$data, 
                              rsds = rsds$data),
                agregation = agregation,
                dates_obs = dates_obs,
                dates_dcpp = rsds$dates,
                model = model,
                method = method,
                fyears = 1,
                config = config,
                ncores = 1,
                step_fun = step)
  
  chunks <- list(lat = 13, lon = 13) #, member = 2)
  job_wallclock <- '05:00:00'
  # chunks <- list(lat = 53, lon = 24) #, member = 2)
  # job_wallclock <- '04:00:00'
  
  output <- Compute(workflow = wf[[1]],
                    chunks = chunks,
                    threads_load = 1,
                    threads_compute = config$ncores,
                    cluster = list(queue_host = 'nord4',
                                   r_module = "R/4.1.2-foss-2019b",
                                   temp_dir = '/gpfs/scratch/bsc32/bsc032693/startR_hpc/',
                                   CDO_module = "CDO/1.9.8-foss-2019b",
                                   ecflow_module = 'ecFlow',
                                   cores_per_job = 9,
                                   job_wallclock = job_wallclock,
                                   max_jobs = config$max_jobs,
                                   special_setup = "nord4",
                                   extra_queue_params = list('#SBATCH -A bsc32', '#SBATCH -q bsc_es'),
                                   bidirectional = FALSE,
                                   polling_period = 10),
                    ecflow_suite_dir = '/home/Earth/smoreno/Desktop/startR_local/',
                    wait = TRUE)
  
} else {stop('retrieve must be TRUE or FALSE')}

saveRDS(object = list(pv_dcpp_DJF = output$pv_dcpp_DJF,
                      pv_dcpp_MAM = output$pv_dcpp_MAM,
                      pv_dcpp_JJA = output$pv_dcpp_JJA,
                      pv_dcpp_SON = output$pv_dcpp_SON,
                      pv_dcpp_Annual = output$pv_dcpp_Annual,
                      pv_dcpp_probs_DJF = output$pv_dcpp_probs_DJF,
                      pv_dcpp_probs_MAM = output$pv_dcpp_probs_MAM,
                      pv_dcpp_probs_JJA = output$pv_dcpp_probs_JJA,
                      pv_dcpp_probs_SON = output$pv_dcpp_probs_SON,
                      pv_dcpp_probs_Annual = output$pv_dcpp_probs_Annual,
                      pv_obs_DJF = output$pv_obs_DJF, 
                      pv_obs_MAM = output$pv_obs_MAM,
                      pv_obs_JJA = output$pv_obs_JJA,
                      pv_obs_SON = output$pv_obs_SON,
                      pv_obs_Annual = output$pv_obs_Annual,
                      n_effective_dcpp_DJF = output$n_effective_dcpp_DJF,
                      n_effective_dcpp_MAM = output$n_effective_dcpp_MAM,
                      n_effective_dcpp_JJA = output$n_effective_dcpp_JJA,
                      n_effective_dcpp_SON = output$n_effective_dcpp_SON,
                      n_effective_dcpp_Annual = output$n_effective_dcpp_Annual,
                      n_effective_obs_DJF = output$n_effective_obs_DJF,
                      n_effective_obs_MAM = output$n_effective_obs_MAM,
                      n_effective_obs_JJA = output$n_effective_obs_JJA,
                      n_effective_obs_SON = output$n_effective_obs_SON,
                      n_effective_obs_Annual = output$n_effective_obs_Annual,
                      lon = lon,
                      lat = lat,
                      config = config,
                      dates = rsds$dates),
        file = paste0(config$capacity_factors_models,'solar_capacity_factor_',model,'_',fyears_string,'_',region,'_',members,'_',method,'.rds'))
# file = paste0(config$capacity_factors_models,'test_solar_cdo.rds'))

print(paste0('Finished in ',round(x = (proc.time() - time_starting_exec)[3]/60, digits = 0),' min.'))
