setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
source('/esarchive/scratch/smoreno/gitlab/capacity_factors/simpl_wind_capacity_factors.R')
# args <- commandArgs(trailingOnly = TRUE)
# model <- args[1]
# IEC_class <- args[2]
# member <- args[3]
models <- config$models
region <- 'Iberia'
fperiods <- '1-3'
model <- 'EC-Earth3-i4'
agregation <- 'seasons' 
retrieve <- FALSE
IEC_class <- 'I'
member <- 'r1i4p1f1'
grid <- config$grid_25
machine <- 'nord4'
job_control <- 'ecflow'

# config$years <- 1966
# config$sdates <- 1965:1967
# config$regions = list('Iberia' = list('lon_min' = 0, 'lon_max' = 0.5, 'lat_min' = 41, 'lat_max' = 41.5))
if (identical(models,'all')){
  models <- c(config$models,config$multimodels)
} else {
  models <- unlist(strsplit(x = models, split = ','))
}
if (identical(fperiods,'all')){
  fperiods <- config$fperiods
} else {
  aux <- unlist(strsplit(x = fperiods, split = ','))
  fperiods <- list()
  for (i in 1:length(aux)){
    auxi <- unlist(strsplit(x = aux[i], split = '-'))
    fperiods[[i]] <- seq(from = auxi[1], to = auxi[length(auxi)], by = 1)
  }
}

  for (fyears in fperiods){
    
    time_starting_exec <- proc.time()
    
    if (length(fyears) == 1){fyears_string <- paste0('fyear',fyears[1])} else {fyears_string <- paste0('fyears',fyears[1],'-',fyears[length(fyears)])}
    sdates <- config$sdates[1] : (config$years[length(config$years)] - as.integer(fyears[length(fyears)]))
    years <- (sdates[1] + as.integer(fyears[1])) : config$years[length(config$years)]

    print(paste0('Starting ',model,' - ',fyears_string))
    
    ################################################################################################
    #################################### Loading the data ##########################################
    ################################################################################################
    
    print('Loading the data...')
    
    time_dim <- 'sdate'
    sys_info <- Sys.info()['nodename']
    machine_index <- substr(sys_info, 1, 2)
      path <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
      path_mask <- '/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/'
      era5_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'  
      # gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/temporal_files/aux/GWA_$var$m_t170grid.nc"
      gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/temporal_files/aux/GWA_$var$m_int_Europe.nc"

    total_hours <- 8928
    desired_hours <- c(0, 6, 12, 18)
    selected_indices <- NULL
    for (hour in desired_hours) {
          selected_indices <- c(selected_indices, seq(hour + 1, total_hours, by = 24))
    }
    selected_indices <- 1:30
      obs_uas <- startR::Start(dat = path, var = 'uas',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', time = selected_indices,
                           time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = retrieve)

      obs_vas <- startR::Start(dat = path, var = 'vas',
                               lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                               lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                               year = paste0(config$years), month = 'all', time = selected_indices,
                               time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                               synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                               return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                               lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                               num_procs = 1, retrieve = retrieve)
    
      dates_obs <- attr(obs_vas,"Variables")$common$time
      lon <- attr(obs_vas,'Variables')$common[['lon']]
      lat <- attr(obs_vas,'Variables')$common[['lat']]
      
      era5_wind <- startR::Start(dat = era5_path, var = 'sfcWind',
                                 lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                 lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                 year = paste0(config$years), month = 'all', time = 'all',
                                 synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                 return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                                 lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                 num_procs = 1, retrieve = retrieve)
      
      
      gwa <- startR::Start(dat = gwa100_file, var = 'WSPD100',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = retrieve)
      
      mask_ori <- startR::Start(dat = paste0(path_mask,'$var$.nc'), var = 'sftof',
                                lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                                lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                num_procs = 1, retrieve = retrieve)
      
      
    if (identical(model,'MRI-ESM2-0')) {
      indices_fyears <- fyears[-which(fyears > 4)]
    } else if (identical(model,'MRI-ESM2-0') & any(fyears > 5)){
      indices_fyears <- fyears[-which(fyears > 5)]
    } else {indices_fyears <- fyears}
      indices_fyears <- 1

      for (fy in fyears){
      # dcpp <- startR_calls_subdaily_wind(model = model,fy = fy, config = config, lon_min = config$regions[[region]]$lon_min, lon_max = config$regions[[region]]$lon_max,
      #                                      lat_min = config$regions[[region]]$lat_min, lat_max = config$regions[[region]]$lat_max, member = member,
      #                                      retrieve = retrieve)
        dcpp <- startR_calls_interpolated_wind(model = model,fy = fy, config = config, region = region, member = member,
                                               retrieve = retrieve)
        uas <- dcpp$uas
        vas <- dcpp$vas
        
        if ((machine == 'amd' | machine == 'mn5') && fy == fyears[1]){
          chars <- apply(attr(obs_uas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
            paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(x, 36, nchar(x)))})
          attr(obs_uas,"ExpectedFiles") <- chars
          obs_uas[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(obs_uas[2], 36, nchar(obs_uas[2])))
          chars <- apply(attr(obs_vas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
            paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(x, 36, nchar(x)))})
          attr(obs_vas,"ExpectedFiles") <- chars
          obs_vas[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(obs_vas[2], 36, nchar(obs_vas[2])))
          chars <- apply(attr(uas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
            paste0("/gpfs/scratch/bsc32/bsc32693/scratch", substr(x, 27, nchar(x)))})
          attr(uas,"ExpectedFiles") <- chars
          uas_expr <- deparse(uas[[2]])
          uas_expr_mod <- gsub(
            "/esarchive/scratch/smoreno",
            "/gpfs/scratch/bsc32/bsc32693/scratch",
            uas_expr
          )
          uas[[2]] <- eval(parse(text = uas_expr_mod))
          chars <- apply(attr(vas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
            paste0("/gpfs/scratch/bsc32/bsc32693/scratch", substr(x, 27, nchar(x)))})
          attr(vas,"ExpectedFiles") <- chars
          vas_expr <- deparse(vas[[2]])
          vas_expr_mod <- gsub(
            "/esarchive/scratch/smoreno",
            "/gpfs/scratch/bsc32/bsc32693/scratch",
            vas_expr
          )
          vas[[2]] <- eval(parse(text = vas_expr_mod))
          chars <- apply(attr(era5_wind,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
            paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5", substr(x, 28, nchar(x)))})
          attr(era5_wind,"ExpectedFiles") <- chars
          era5_wind[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/", substr(era5_wind[2], 24, nchar(era5_wind[2])))
          chars <- apply(attr(gwa,"ExpectedFiles"), c(1), FUN = function(x) {
            paste0("/gpfs/scratch/bsc32/bsc32693/scratch/", substr(x, 28, nchar(x)))})
          attr(gwa,"ExpectedFiles") <- chars
          gwa[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/", substr(gwa[2], 28, nchar(gwa[2])))
          chars <- apply(attr(mask_ori,"ExpectedFiles"), c(1), FUN = function(x) {
            paste0("/gpfs/scratch/bsc32/bsc32693/scratch/capacity_outputs/temporal_files/aux/", substr(x, 55, nchar(x)))})
          attr(mask_ori,"ExpectedFiles") <- chars
          mask_ori[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/capacity_outputs/temporal_files/aux/", substr(mask_ori[2], 55, nchar(mask_ori[2])))
        }
      
          output_dims <- list(cf_dcpp_DJF = 'sdate',
                              cf_dcpp_MAM = 'sdate',
                              cf_dcpp_JJA = 'sdate',
                              cf_dcpp_SON = 'sdate',
                              cf_dcpp_Annual = 'sdate',
                              cf_obs_DJF = 'sdate',
                              cf_obs_MAM = 'sdate',
                              cf_obs_JJA = 'sdate',
                              cf_obs_SON = 'sdate',
                              cf_obs_Annual = 'sdate',
                              cf_probs_DJF = c('bin','sdate'),
                              cf_probs_MAM = c('bin','sdate'),
                              cf_probs_JJA = c('bin','sdate'),
                              cf_probs_SON = c('bin','sdate'),
                              cf_probs_Annual = c('bin','sdate'),
                              subdays_dcpp_DJF = 'sdate',
                              subdays_dcpp_MAM = 'sdate',
                              subdays_dcpp_JJA = 'sdate',
                              subdays_dcpp_SON = 'sdate',
                              subdays_dcpp_Annual = 'sdate',
                              subdays_obs_DJF = 'sdate',
                              subdays_obs_MAM = 'sdate',
                              subdays_obs_JJA = 'sdate',
                              subdays_obs_SON = 'sdate',
                              subdays_obs_Annual = 'sdate'
          )

          target_dims <-  list(obs_uas = c('year', 'time'),
                               obs_vas = c('year', 'time'),
                               dcpp_uas = c('sdate','time','member','exp','initmonth'),
                               dcpp_vas = c('sdate','time','member','exp','initmonth'),
                               era5_wind = c('year','month'),
                               mask_ori = c('var'),
                               gwa_spd = c('var'))
          
          if (isTRUE(retrieve)) {
            output <- multiApply::Apply(data = list(obs_uas = obs_uas,
                                                    obs_vas = obs_vas,
                                                    dcpp_uas = uas,
                                                    dcpp_vas = vas,
                                                    era5_wind = era5_wind,
                                                    mask_ori = mask_ori, #InsertDim(data = mask_ori[,,1:3,1:3], posdim = 1, lendim = 1, name = 'var'),
                                                    gwa_spd = gwa #InsertDim(data = gwa[,,1:3,1:3], posdim = 1, lendim = 1, name = 'var')
                                                    ),
                                        agregation = agregation,
                                        dates_obs = dates_obs,
                                        dates_dcpp = dcpp$dates,
                                        fyears = fy,
                                        IEC_class = 'I',
                                        model = model,
                                        machine = machine,
                                        output_dims = output_dims, 
                                        target_dims = target_dims, 
                                        fun = simpl_wind_capacity_factors, 
                                        config = config, 
                                        ncores = config$ncores)

          } else if (isFALSE(retrieve)) {
            
            step <- startR::Step(fun = simpl_wind_capacity_factors,
                                 target_dims = target_dims, 
                                 output_dims = output_dims)
            
            wf <- startR::AddStep(inputs = list(obs_uas = obs_uas, 
                                                obs_vas = obs_vas,
                                                dcpp_uas = uas, 
                                                dcpp_vas = vas,
                                                era5_wind = era5_wind,
                                                mask_ori = mask_ori,
                                                gwa_spd = gwa),
                                  agregation = agregation,
                                  dates_obs = dates_obs,
                                  dates_dcpp = dcpp$dates,
                                  fyears = fy,
                                  IEC_class = 'I',
                                  model = model,
                                  machine = machine,
                                  config = config,
                                  ncores = config$ncores,
                                  step_fun = step)
            

            # chunks <- list(lat = 26, lon = 23) 
            chunks <- list(lat = length(lat), lon = length(lon))
            config$ncores <- 5
            
            if (machine == 'nord4'){
              special_setup <- 'nord4'
              max_jobs <- 300
              conda_env <- NULL
              job_wallclock <- '12:00:00'
            } else if (machine == 'amd'){
              special_setup <- 'none'
              max_jobs <- 210
              conda_env <- NULL
              job_wallclock <- '06:00:00'
            } else if (machine == 'mn5'){
              special_setup <- 'none'
              max_jobs <- 300
              conda_env <- "/gpfs/projects/bsc32/repository/apps/conda_envs/SUNSET-env_2.0.0"
            }
            
            if (job_control == 'ecflow'){
              output <- Compute(workflow = wf[[1]],
                                chunks = chunks,
                                threads_load = 1,
                                threads_compute = config$ncores,
                                cluster = list(queue_host = machine, #'nord4',
                                               r_module = "R/4.1.2-foss-2019b",
                                               temp_dir = '/gpfs/scratch/bsc32/bsc032693/startR_hpc/',
                                               conda_env = conda_env,
                                               CDO_module = "CDO/1.9.8-foss-2019b",
                                               ecflow_module = 'ecFlow',
                                               cores_per_job = 5,
                                               job_wallclock = job_wallclock,
                                               max_jobs = max_jobs,
                                               special_setup = special_setup,
                                               extra_queue_params = list('#SBATCH -A bsc32', '#SBATCH -q bsc_es'),
                                               bidirectional = FALSE,
                                               polling_period = 10),
                                ecflow_suite_dir = '/home/Earth/smoreno/Desktop/startR_local/',
                                wait = TRUE)
              
            } else if (job_control == 'autosubmit'){
              Compute(workflow = wf,
                      chunks = chunks,
                      threads_load = 1,
                      threads_compute = config$ncores,
                      cluster = list(
                        queue_host = machine,
                        expid = "a9gq",
                        hpc_user = "bsc032693",
                        conda_env = conda_env,
                        autosubmit_module = 'autosubmit/4.0.98-foss-2015a-Python-3.7.3',
                        # autosubmit_module = 'autosubmit/4.0.98-foss-2021b-Python-3.9.6',
                        cores_per_job = 1,
                        job_wallclock = job_wallclock,
                        max_jobs = 4,
                        special_setup = special_setup
                      ),
                      workflow_manager = 'autosubmit',
                      autosubmit_suite_dir = "/home/Earth/smoreno/startR_local_autosubmit/",
                      autosubmit_server = 'bscesautosubmit01',
                      wait = TRUE
              )
            }

            
          } else {stop('retrieve must be TRUE or FALSE')}
          
          
          saveRDS(object = list(cf_dcpp_DJF = output$cf_dcpp_DJF,
                                cf_dcpp_MAM = output$cf_dcpp_MAM,
                                cf_dcpp_JJA = output$cf_dcpp_JJA,
                                cf_dcpp_SON = output$cf_dcpp_SON,
                                cf_dcpp_Annual = output$cf_dcpp_Annual,
                                cf_obs_DJF = output$cf_obs_DJF,
                                cf_obs_MAM = output$cf_obs_MAM,
                                cf_obs_JJA = output$cf_obs_JJA,
                                cf_obs_SON = output$cf_obs_SON,
                                cf_obs_Annual = output$cf_obs_Annual,
                                cf_probs_DJF = output$cf_probs_DJF,
                                cf_probs_MAM = output$cf_probs_MAM,
                                cf_probs_JJA = output$cf_probs_JJA,
                                cf_probs_SON = output$cf_probs_SON,
                                cf_probs_Annual = output$cf_probs_Annual,
                                subdays_dcpp_DJF = output$subdays_dcpp_DJF,
                                subdays_dcpp_MAM = output$subdays_dcpp_MAM,
                                subdays_dcpp_JJA = output$subdays_dcpp_JJA,
                                subdays_dcpp_SON = output$subdays_dcpp_SON,
                                subdays_dcpp_Annual = output$subdays_dcpp_Annual,
                                subdays_obs_DJF = output$subdays_obs_DJF,
                                subdays_obs_MAM = output$subdays_obs_MAM,
                                subdays_obs_JJA = output$subdays_obs_JJA,
                                subdays_obs_SON = output$subdays_obs_SON,
                                subdays_obs_Annual = output$subdays_obs_Annual,
                                lon = lon,
                                lat = lat,
                                config = config,
                                dates = dcpp$dates),
                  file = paste0(config$capacity_factors_models,'wind_capacity_factor_',model,'_fyear_',fy,'_',member,'_',region,'_IEC_class_',IEC_class,'.rds'))
      }
  }

    print(paste0('Finished in ',round(x = (proc.time() - time_starting_exec)[3]/60, digits = 0),' min.'))
  
