library(ncdf4)
library(RNetCDF)
source('config.R')
variables <- c('uas','vas')
sys_info <- Sys.info()['nodename']
machine_index <- substr(sys_info, 1, 2)


region <- 'Europe'
crop_region <- c(config$regions[[region]]$lon_min,
                 config$regions[[region]]$lon_max,
                 config$regions[[region]]$lat_min,
                 config$regions[[region]]$lat_max)

crop_files <- function(variable, output_dir, crop_region, region_name) {
  input_dir <- file.path('/esarchive/recon/ecmwf/era5/1hourly', 
                         paste0(variable, '-r1440x721cds'))
  
  if (!dir.exists(output_dir)) {
    dir.create(output_dir, recursive = TRUE)
  }
  
  nc_files <- list.files(input_dir, pattern = "\\.nc$", full.names = TRUE)
  
  for (file in nc_files) {
    filename <- basename(file)
    year_match <- regmatches(filename, regexpr("\\d{4}", filename))
    year <- as.numeric(year_match)
    if (!is.na(year) && year >= 1960 && year <= 2021) {
      
      nc <- nc_open(file)
      dim_names <- names(nc$dim)
      lon_name <- if ("lon" %in% dim_names) "lon" else if ("longitude" %in% dim_names) "longitude" else stop("No se encuentra lon/longitude")
      lat_name <- if ("lat" %in% dim_names) "lat" else if ("latitude" %in% dim_names) "latitude" else stop("No se encuentra lat/latitude")

      lon <- ncvar_get(nc, lon_name)
      lat <- ncvar_get(nc, lat_name)
      time <- ncvar_get(nc, "time")
      time_units <- nc$dim$time$units

      lon_idx <- which(lon >= crop_region[1] & lon <= crop_region[2])
      lat_idx <- which(lat >= crop_region[3] & lat <= crop_region[4])

      data <- ncvar_get(nc, variable, 
                        start = c(min(lon_idx), min(lat_idx), 1), 
                        count = c(length(lon_idx), length(lat_idx), length(time)))
      nc_close(nc)

      dim_lon <- ncdim_def(lon_name, "degrees_east", lon[lon_idx])
      dim_lat <- ncdim_def(lat_name, "degrees_north", lat[lat_idx])
      dim_time <- ncdim_def("time", time_units, time)
      var_def <- ncvar_def(variable, "m/s", list(dim_lon, dim_lat, dim_time), 
                           missval = 1e20, prec = "float")
      var_def <- ncvar_def(variable, "m/s", list(dim_lon, dim_lat, dim_time), 
                           missval = 1e20, prec = "float")
      out_file <- file.path(output_dir, basename(file))
      nc_out <- nc_create(out_file, var_def)
      ncvar_put(nc_out, variable, data)
      nc_close(nc_out)
    }
  }
}

for (variable in variables){
  if (machine_index %in% c('gl', 'as', 'gs', 'da')) {
    output_dir <- paste0('/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly/', variable, '-r1440x721cds/')
  } else {
    output_dir <- paste0('/esarchive/scratch/smoreno/recon/era5/1hourly/', variable, '-r1440x721cds/')
  }
  crop_files(variable, output_dir, crop_region, region)
}

