setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors')
library(CSTools)
library(ClimProjDiags)
library(s2dv)
source('config.R')

path_models <- '/esarchive/exp/ecearth/cmip6-$exp$/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/$exp$/$member$/6hrPlevPt/$var$/gr/v20210910/$var$_6hrPlevPt_EC-Earth3_$exp$_s$sdate$$initmonth$-$member$_gr_$aux$.nc'
path_obs <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
path_obs_100 <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
path_mask <- '/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/'
era5_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'  
gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/GWA_$var$m_t170grid.nc"
calendar <- '6hr'
month_init <- 'Nov'
config$sdates <- 2023
config$years <- 2024
exp_list <- list(); for (sdt in config$sdates){if (sdt <= 2020){exp_list[[as.character(sdt)]] <- 'dcppA-hindcast'} else {exp_list[[as.character(sdt)]] <- 'dcppB-forecast'}}
initmonth_list <- list(); for (sdt in config$sdates){if (sdt <= 2023){initmonth_list[[as.character(sdt)]] <- ''} else {initmonth_list[[as.character(sdt)]] <- '1101'}}
times <- get_fdays(fyears = 1, calendar = calendar, month_init = month_init)
member <- 'r1i4p1f1'
retrieve <- TRUE
region <- 'zgz'
year <- 2024


uas <- startR::Start(
  dat = list(list(path = path_models,pattern_dims = list(exp = "exp",member = "member",var = "var",sdate = "sdate",aux = "aux"))),
  var = 'uas',
  lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
  lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
  sdate = paste(config$sdates),time = indices(times),
  member = member,exp = exp_list,aux = "all",initmonth = initmonth_list, initmonth_depends = 'sdate',aux_depends = "sdate",
  exp_depends = "sdate",time_across = "aux",merge_across_dims = TRUE,synonims = list(lat = c('lat', 'latitude'), lon = c('lon', 'longitude'), time = c('time')),
  return_vars = list(lat = NULL, lon = NULL, time = c('sdate')),lon_var = 'lon',lat_var = 'lat',lat_reorder = Sort(decreasing = FALSE),
  lon_reorder = CircularSort(-180, 180),transform = CDORemapper,transform_extra_cells = 2,
  transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/uas-r1440x721cds/uas_199001.nc'),method = 'bil'),transform_vars = c('lat', 'lon'),
  num_procs = 1,retrieve = retrieve
)

vas <- startR::Start(
  dat = list(list(path = path_models,pattern_dims = list(exp = "exp",member = "member",var = "var",sdate = "sdate",aux = "aux"))),
  var = 'vas',  
  lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
  lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
  sdate = paste(config$sdates),time = indices(times),
  member = member,exp = exp_list,aux = "all",initmonth = initmonth_list, initmonth_depends = 'sdate',aux_depends = "sdate",
  exp_depends = "sdate",time_across = "aux",merge_across_dims = TRUE,synonims = list(lat = c('lat', 'latitude'), lon = c('lon', 'longitude'), time = c('time')),
  return_vars = list(lat = NULL, lon = NULL, time = c('sdate')),lon_var = 'lon',lat_var = 'lat',lat_reorder = Sort(decreasing = FALSE),
  lon_reorder = CircularSort(-180, 180),transform = CDORemapper,transform_extra_cells = 2,
  transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/vas-r1440x721cds/vas_199001.nc'),method = 'bil'),transform_vars = c('lat', 'lon'),
  num_procs = 1,retrieve = retrieve
)

dates <- attr(vas,"Variables")$common$time

total_hours <- 8928
desired_hours <- c(0, 6, 12, 18)
selected_indices <- NULL
for (hour in desired_hours) {
  selected_indices <- c(selected_indices, seq(hour + 1, total_hours, by = 24))
}
obs_uas <-  startR::Start(dat = path_obs, var = 'uas',
                          lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                          lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                          year = paste0(config$years), month = 'all', time = selected_indices,
                          time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)

obs_vas <- startR::Start(dat = path_obs, var = 'vas',  
                         lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                         lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                         year = paste0(config$years), month = 'all', time = selected_indices,
                         time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                         lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                         num_procs = 1, retrieve = retrieve)


obs_uas_100 <-  startR::Start(dat = path_obs_100, var = 'u100m',
                          lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                          lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                          year = paste0(config$years), month = 'all', time = selected_indices,
                          time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)

obs_vas_100 <- startR::Start(dat = path_obs_100, var = 'v100m',  
                         lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                         lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                         year = paste0(config$years), month = 'all', time = selected_indices,
                         time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                         lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                         num_procs = 1, retrieve = retrieve)



lon <- attr(obs_vas,'Variables')$common[['lon']]
lat <- attr(obs_vas,'Variables')$common[['lat']]
dates_obs <- attr(obs_vas,"Variables")$common$time

era5_wind <- startR::Start(dat = era5_path, var = 'sfcWind',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', time = 'all',
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           transform = CDORemapper,
                           transform_extra_cells = 2,
                           transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/uas-r1440x721cds/uas_199001.nc'),
                                                   method = 'bil'),
                           transform_vars = c('lat', 'lon'),
                           num_procs = 1, retrieve = retrieve)

gwa <- startR::Start(dat = gwa100_file, var = 'WSPD100',
                     lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                     lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                     synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                     return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                     lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                     transform = CDORemapper,
                     transform_extra_cells = 2,
                     transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/uas-r1440x721cds/uas_199001.nc'),
                                             method = 'bil'),
                     transform_vars = c('lat', 'lon'),
                     num_procs = 1, retrieve = retrieve)

mask_ori <- startR::Start(dat = paste0(path_mask,'$var$.nc'), var = 'sftof',
                          lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                          lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          transform = CDORemapper,
                          transform_extra_cells = 2,
                          transform_params = list(grid = paste0('/esarchive/recon/ecmwf/era5/1hourly/uas-r1440x721cds/uas_199001.nc'),
                                                  method = 'bil'),
                          transform_vars = c('lat', 'lon'),
                          num_procs = 1, retrieve = retrieve)

sfc_wind_dcpp <- sqrt(uas^2 + vas^2)
sfc_wind_obs <- sqrt(obs_uas^2 + obs_vas^2)
wind_100 <- sqrt(obs_uas_100^2 + obs_vas_100^2)
sfc_wind_obs <- drop(sfc_wind_obs)
wind_100 <- drop(wind_100)
# dat <- dates_obs[1,]
# indices_remove <- which(is.na(dat))
# indices_to_remove <- which(sapply(indices_remove, function(x) !is.null(x) && x > 0))
# if (config$years %% 4 == 0 && (config$years %% 100 != 0 || config$years %% 400 == 0)){
#   feb29_indices <- (59 * 4) + 1:4
#   indices_remove <- c(indices_remove, feb29_indices)
# } 
# keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove)
# sfc_wind_obs <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'time', indices = keep_indices, drop = FALSE)

time_has_NA <- sapply(seq_len(dim(obs_uas)["time"]), function(t) {
  any(is.na(obs_uas[,,,,,t]))
})
keep_indices <- !time_has_NA
if (config$years %% 4 == 0 && (config$years %% 100 != 0 || config$years %% 400 == 0)){
  feb29_indices <- (59 * 4) + (1:4)
  keep_indices[feb29_indices] <- FALSE
}
sfc_wind_obs <- ClimProjDiags::Subset(x = sfc_wind_obs, along = "time", indices = keep_indices, drop = FALSE)
wind_100 <- ClimProjDiags::Subset(x = wind_100, along = "time", indices = keep_indices, drop = FALSE)


sfc_wind_dcpp <- drop(sfc_wind_dcpp)
era5_wind <- s2dv::MeanDims(data = era5_wind, dims = c('year','month'))
gwa <- drop(gwa)
era5_wind <- drop(era5_wind)
ratio <- gwa/era5_wind
ratio_expanded <- array(ratio, dim = dim(sfc_wind_obs))
sfc_wind_obs <- sfc_wind_obs * ratio_expanded
multiplication <- ifelse(mask_ori == 0, 1.39, 1.29)
multiplication <- drop(multiplication)
multiplication_exp <- array(multiplication, dim = dim(sfc_wind_obs))
sfc_wind_dcpp <- sfc_wind_dcpp * multiplication_exp
sfc_wind_obs <- sfc_wind_obs * multiplication_exp

saveRDS(object = list(sfc_wind_dcpp = sfc_wind_dcpp,
                      sfc_wind_obs = sfc_wind_obs,
                      wind_100 = wind_100,
                      lon = lon,
                      lat = lat,
                      config = config,
                      dates = dates),
        file = paste0(config$capacity_factors_models,'wind_evolution_',region,'_',year,'.rds'))

sfc_wind_dcpp_line <- sfc_wind_dcpp[2, 2, ]
sfc_wind_obs_line <- sfc_wind_obs[2, 2, ]
wind_100_line <- wind_100[2, 2, ]
n <- length(sfc_wind_obs_line)
x <- 1:n
percent_obs <- sum(sfc_wind_obs_line > 2 & sfc_wind_obs_line < 25) / n * 100
percent_obs_100 <- sum(wind_100_line > 2 & wind_100_line < 25) / n * 100
percent_dcpp <- sum(sfc_wind_dcpp_line > 2 & sfc_wind_dcpp_line < 25) / n * 100
region <- 'Zaragoza'
dates <- seq(as.Date(paste0(year, "-01-01")), by = "7 days", length.out = ceiling(n / (7 * 4)))
date_positions <- seq(1, n, by = 7 * 4)
output_file <- paste0('Wind_evolution_EC-Earth3-i4_', region, '_', year, '.png')
png(output_file, width = 1200, height = 800, res = 150)
max_y <- max(c(sfc_wind_obs_line, wind_100_line, sfc_wind_dcpp_line), na.rm = TRUE) + 2
plot(x, sfc_wind_obs_line, type = "l", col = "blue", lwd = 2, ylim = c(0, max_y),
     xlab = "Date", ylab = "Wind speed (m/s)", xaxt = "n",
     main = paste0("Wind Time Series (", region, ", ", year, ")"))
lines(x, wind_100_line, col = "green", lwd = 2)
lines(x, sfc_wind_dcpp_line, col = "red", lwd = 2)
axis(1, at = date_positions, labels = format(dates, "%d %b"), cex.axis = 0.8, las = 2)
rect(0, 0, n, 2, col = rgb(0.2, 0.2, 0.2, alpha = 0.5), border = NA)
rect(0, 25, n, max_y, col = rgb(0.2, 0.2, 0.2, alpha = 0.5), border = NA)
legend("topright",
       legend = c(
         paste0("Obs surface (Eff. = ", round(percent_obs, 1), "%)"),
         paste0("Obs 100m (Eff. = ", round(percent_obs_100, 1), "%)"),
         paste0("EC-Earth3-i4 (Eff. = ", round(percent_dcpp, 1), "%)")
       ),
       col = c("blue", "green", "red"), lwd = 2)
dev.off()







