source('config.R')
region <- 'Iberia'
retrieve <- FALSE
path <- '/esarchive/recon/ecmwf/era5/hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'

data <- startR::Start(dat = path, var = 'rsds', 
                      # lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                      # lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                      lon = values(list(1,1.2)),lat = values(list(40,40.2)),
                      year = paste0(2000:2001), month = 'all', time = 'all',
                      time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = FALSE,
                      synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                      return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                      lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                      num_procs = 1, retrieve = retrieve)

dates <- attr(data,'Variables')$common$time
target_dims <- c('year','time')

output_dims <- list(direct_djf = c('year','time'), hr3_djf = c('year','time'), 
                    direct_mam = c('year','time'), hr3_mam = c('year','time'), direct_jja = c('year','time'),
                    hr3_jja = c('year','time'), direct_son = c('year','time'), 
                    hr3_son = c('year','time'), direct = c('year','time'), hr3 = c('year','time'))

if (isTRUE(config$retrieve)) {
  
  output <- multiApply::Apply(data = data, target_dims = target_dims, output_dims = output_dims, fun = daily_means_3hr, 
                              dates = dates, config = config, ncores = 1)
  
} else if (isFALSE(config$retrieve)) {
  
  step <- startR::Step(fun = daily_means_3hr,
                       target_dims = target_dims, 
                       output_dims = output_dims)
  
  wf <- startR::AddStep(inputs = list(data = data),
                        dates = dates,
                        config = config,
                        ncores = config$ncores,
                        step_fun = step)
  
  chunks <- list(lat = 1, lon = 1) #, member = 2)
  job_wallclock <- '01:00:00'
  
  output <- Compute(workflow = wf[[1]],
                    chunks = chunks,
                    threads_load = 1,
                    threads_compute = config$ncores,
                    # cluster = list(queue_host = 'nord4',
                    #                r_module = "R/4.1.2-foss-2019b",
                    #                temp_dir = '/gpfs/scratch/bsc32/bsc032693/startR_hpc/',
                    #                CDO_module = "CDO/1.9.8-foss-2019b",
                    #                ecflow_module = 'ecFlow',
                    #                cores_per_job = 9,
                    #                job_wallclock = job_wallclock,
                    #                max_jobs = config$max_jobs,
                    #                special_setup = "nord4",
                    #                extra_queue_params = list('#SBATCH -A bsc32', '#SBATCH -q bsc_es'),
                    #                bidirectional = FALSE,
                    #                polling_period = 10),
                    ecflow_suite_dir = '/home/Earth/smoreno/Desktop/startR_local/',
                    wait = TRUE)
}