library(parallel)
source('config.R')
region <- 'Europe'
crop_region <- c(
  config$regions[[region]]$lon_min, config$regions[[region]]$lon_max,
  config$regions[[region]]$lat_min, config$regions[[region]]$lat_max
)
output_dir_base <- "/esarchive/scratch/smoreno/capacity_outputs/temporal_files/aux"
grid <- '/esarchive/scratch/smoreno/capacity_outputs/temporal_files/uas/EC-Earth3-i4/r1i4p1f1/final_Europe_uas_6hrPlevPt_EC-Earth3_dcppA-hindcast_s2018-r1i4p1f1_gr_202011010000-202110311800.nc'

interpolate_and_save <- function(input_file, 
                                 output_dir, 
                                 output_name, 
                                 crop_region,
                                 grid) {
  if (!dir.exists(output_dir)) dir.create(output_dir, recursive = TRUE)
  
  temp_file <- file.path(output_dir, paste0("remap_", basename(input_file)))
  output_file <- file.path(output_dir, output_name)
  
  message("Procesando archivo: ", basename(input_file))
  
  remap_cmd <- paste0("cdo remapbil,", grid, " ", shQuote(input_file), " ", shQuote(temp_file))
  system(remap_cmd)
  
  crop_cmd <- paste0("cdo sellonlatbox,", paste(crop_region, collapse = ","), " ", shQuote(temp_file), " ", shQuote(output_file))
  system(crop_cmd)
  
  file.remove(temp_file)
  message("Guardado en: ", output_file)
}

gwa_file <- "/esarchive/scratch/smoreno/capacity_outputs/GWA_WSPD100m_t170grid.nc"
gwa_output_name <- "GWA_WSPD100m_int_Europe.nc"
interpolate_and_save(gwa_file, output_dir_base, gwa_output_name, crop_region, grid)
sdfdsfd
years <- config$years
months <- sprintf("%02d", 1:12)
var <- "sfcWind"

for (year in years) {
  for (month in months) {
    era5_file <- paste0("/esarchive/recon/ecmwf/era5/monthly_mean/", var, "_f1h-r1440x721cds/", var, "_", year, month, ".nc")
    if (file.exists(era5_file)) {
      era5_output_name <- paste0(var, "_", year, month, "_int_Europe.nc")
      interpolate_and_save(era5_file, output_dir_base, era5_output_name, crop_region, grid)
    } else {
      message("No existe el archivo: ", era5_file)
    }
  }
}

mask_file <- "/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/sftof.nc"
mask_output_name <- "sftof_int_Europe.nc"
interpolate_and_save(mask_file, output_dir_base, mask_output_name, crop_region, grid)
