make_eval_train_dexes_mine <- function(eval.method, amt.points, amt.points_cor,
         k = 1, tail.out = TRUE) {
  # eval.method: 'leave-one-out', 'k-fold', 'in-sample', "hindcast-vs-forecast", "retrospective"
  # amt.points: length of the sample
  # amt.points_cor: only needed for hindcast-vs-forecast method
  # k: the number of samples to leave-out
  # tail_out: boolean for 'k-fold method; TRUE to remove both extremes of the sample when k is 
  # in the extreme keeping the same sample size for all k-folds (e.g. amt.points=50, k=3, 
  # eval.dexes=1, train.dexes={3,49}). FALSE to remove only the corresponding tail (e.g..
  # amt.points=50, k=3, eval.dexes=1, train.dexes={3,50})
  
  
  if (k >= amt.points && !is.null(k)) { 
    stop("k need to be smaller than the amt.points")
  }
  
  if (eval.method == "leave-one-out") {
    dexes.lst <- lapply(seq(1, amt.points), function(x) return(list(eval.dexes = x,
                                                                    train.dexes = seq(1, amt.points)[-x])))
  } else if (eval.method == "k-fold") {
    # k is a odd number
    if (tail.out == TRUE){
      dexes.lst <- lapply(seq(1, amt.points), function(x, kfold = k) {
        if (x >= ((kfold-1)/2) + 1 && x + ((kfold-1)/2) <= amt.points) {
          ind <- (x-((kfold-1)/2)):(x+((kfold-1)/2))
        } else if (x < ((kfold-1)/2) + 1) {
          ind <- c((amt.points - ((kfold-1)/2-x)):amt.points, 1:(x+(kfold-1)/2))
        } else if ((x+((kfold-1)/2)) > amt.points) {
          ind <- c((x-(kfold-1)/2):amt.points, 1:(((kfold-1)/2)-amt.points+x)) 
        } else {
          stop("Review make.eval.train.dexes function")
        }
        return(list(eval.dexes = x,  train.dexes = seq(1, amt.points)[-ind]))
      })
    } else {
      if (k != 1){
        k <- (k-1)/2
      }
      
      dexes.lst <- lapply(seq(1, amt.points), function(x) {
        start_idx <- max(1, x - k)
        end_idx <- min(amt.points, x + k)
        eval_range <- start_idx:end_idx
        train_idx <- setdiff(seq(1, amt.points), eval_range)
        return(list(eval.dexes = x, train.dexes = train_idx))
      })
    }
    
  } else if (eval.method == "retrospective") {
    # k can be any integer indicating the when to start
    dexes.lst <- base::Filter(length, lapply(seq(1, amt.points),
                                             function(x, mindata = k) {
                                               if (x > k) {
                                                 eval.dexes <- x
                                                 train.dexes <- 1:(x-1)
                                                 return(list(eval.dexes = x,
                                                             train.dexes = 1:(x-1)))
                                               }}))
  } else if (eval.method == "in-sample") {
    dexes.lst <- list(list(eval.dexes = seq(1, amt.points), 
                           train.dexes = seq(1, amt.points)))
  } else if (eval.method == "hindcast-vs-forecast") {
    dexes.lst <- list(list(eval.dexes = seq(1,amt.points_cor),
                           train.dexes = seq(1, amt.points)))
  } else {
    stop(paste0("unknown sampling method: ", eval.method))
  }
  return(dexes.lst)
}
