setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')

args <- commandArgs(trailingOnly = TRUE)
# model <- 'BCC-CSM2-MR'
models <- 'EC-Earth3-i4'
region <- 'Iberia'
fyear <- 'fyear1'
plot <- 'obs'
seasons <- config$seasons
letras <- c('a)','b)','c)','d)','e)')
seasons_names <- c('Annual','Winter','Spring','Summer','Autumn')
energy <- 'solar'
fyears_string <- 'fyear1'
IEC_class <- 'I'

for (model in models){

if (energy == 'solar'){
  data <- readRDS(file = paste0(config$metrics_capacity, 'metrics_solar_CF_', model, '_', fyears_string, '_', region, '.rds'))
} else if (energy == 'wind'){
  data <- readRDS(file = paste0(config$metrics_capacity, 'metrics_',energy,'_CF_', model, '_', fyears_string, '_', region, '_IEC_class_',IEC_class,'.rds'))
}

## RESULTS PV
blues_soft <- colorRampPalette(c("#e0f3ff", "#78bced", "#1e6ec8"))
if (energy == 'solar'){
  results_list <- lapply(seasons, function(season) {
    data$results_multimodel[[season]]$pv_multimodel
  })
  brks <- seq(5,25,by = 2)
  cols <- blues_soft(10)
  filled.oceans <- 'gray'
} else if (energy == 'wind'){
  results_list <- lapply(seasons, function(season) {
    data$results_multimodel[[season]]$pv_multimodel*100
  })
  brks <- seq(15, 70, by = 5)
  cols <- blues_soft(11)
  filled.oceans <- FALSE
}

PlotLayout(
  fun = PlotEquiMap, 
  plot_dims = c('lon', 'lat'), 
  ncol = 5, 
  var = results_list, 
  # special_args = acc_sign_list,
  fileout = paste0(config$figures_path,'pv_',energy,'_dcpp_seasons_',model,'.png'), 
  lon = data$lon, 
  lat = data$lat, 
  filled.continents = FALSE, 
  filled.oceans = filled.oceans,
  titles = paste(letras, ' ',seasons_names),
  brks = brks,
  bar_extra_labels = brks,
  cols = cols,
  colNA = 'gray',
  col_inf = "#f5fbff", col_sup = "#1659aa",
  units = '%',
  width = 18,
  height = 5,
  triangle_ends = c(T,T)
)

## ACC PV
acc_value_list <- lapply(seasons, function(season) {
  data$metrics[[season]]$cf$acc$r
})
acc_sign_list <- lapply(seasons, function(season) {
  data$metrics[[season]]$cf$acc$sign
})
brks <- seq(-0.9,0.9,by = 0.1)
special_args <- list()
prop_list <- list()
for (i in 1:length(acc_sign_list)) {
  special_args[[i]] <- list(dots = !acc_sign_list[[i]])  # Aquí acc_sign_list[[i]] es un array
  prop <- significant_fraction(skill = acc_sign_list[[i]],
                               sign = acc_sign_list[[i]], 
                               lon = data$lon, 
                               lat = data$lat, 
                               positive = TRUE, 
                               mask_ocean = TRUE, 
                               mask_grid = config$grid_25, 
                               londim = 'lon', 
                               latdim = 'lat')
  prop <- prop*100
  prop <- round(prop, 1)
  prop_list <- append(prop_list, prop)
}
titles <- paste(letras, ' ',seasons_names,' (',prop_list,'%)', sep= '')
PlotLayout(
  fun = PlotEquiMap, 
  plot_dims = c('lon', 'lat'), 
  ncol = 5, 
  var = acc_value_list, 
  special_args = special_args,
  fileout = paste0(config$figures_path,'acc_',energy,'_seasons_',model,'.png'), 
  lon = data$lon, 
  lat = data$lat, 
  filled.continents = FALSE, 
  filled.oceans = filled.oceans,
  colNA = 'gray',
  titles = titles,
  brks = brks,
  bar_extra_labels = brks,
  width = 18,
  height = 5,
  triangle_ends = c(T,T)
)

## RPSS PV
# rpss_value_list <- lapply(seasons, function(season) {
#   data$metrics[[season]]$cf$rpss$rpss
# })
# rpss_sign_list <- lapply(seasons, function(season) {
#   data$metrics[[season]]$cf$rpss$sign
# })
# brks <- seq(-0.9,0.9,by = 0.1)
# special_args <- list()
# prop_list <- list()
# for (i in 1:length(rpss_sign_list)) {
#   special_args[[i]] <- list(dots = !rpss_sign_list[[i]])  # Aquí acc_sign_list[[i]] es un array
#   prop <- significant_fraction(skill = rpss_value_list[[i]],
#                                sign = rpss_sign_list[[i]], 
#                                lon = data$lon, 
#                                lat = data$lat, 
#                                positive = TRUE, 
#                                mask_ocean = TRUE, 
#                                mask_grid = config$grid_25, 
#                                londim = 'lon', 
#                                latdim = 'lat')
#   prop <- prop*100
#   prop <- round(prop, 1)
#   prop_list <- append(prop_list, prop)
# }
# titles <- paste(letras, ' ',seasons_names,' (',prop_list,'%)', sep= '')
# PlotLayout(
#   fun = PlotEquiMap, 
#   plot_dims = c('lon', 'lat'), 
#   ncol = 5, 
#   var = rpss_value_list, 
#   special_args = special_args,
#   fileout = paste0(config$figures_path,'rpss_',energy,'_seasons_',model,'.png'), 
#   lon = data$lon, 
#   lat = data$lat, 
#   filled.continents = FALSE, 
#   filled.oceans = filled.oceans,
#   colNA = 'gray',
#   titles = titles,
#   brks = brks,
#   bar_extra_labels = brks,
#   width = 18,
#   height = 5,
#   triangle_ends = c(T,T)
# )

## RESULTS N_EFE
if (energy == 'solar'){
  n_efe_list <- lapply(seasons, function(season) {
    raw_vals <- data$results_multimodel[[season]]$n_efe_multimodel
    divisor <- if (season == "Annual") 365 else 91
    percent_vals <- 100 * raw_vals / divisor
    percent_vals
  })
  brks <- seq(5,95,by = 10)
  cols <- blues_soft(9)
} else if (energy == 'wind'){
  n_efe_list <- lapply(seasons, function(season) {
    raw_vals <- data$results_multimodel[[season]]$n_efe_multimodel
    divisor <- if (season == "Annual") 1480 else 370
    percent_vals <- 100 * raw_vals / divisor
    percent_vals[percent_vals == 0] <- NA
    percent_vals
  })
  
  brks <- seq(85,95,by = 1)
  cols <- blues_soft(10)
}

blues_soft <- colorRampPalette(c("#e0f3ff", "#78bced", "#1e6ec8"))
PlotLayout(
  fun = PlotEquiMap, 
  plot_dims = c('lon', 'lat'), 
  ncol = 5, 
  var = n_efe_list, 
  # special_args = acc_sign_list,
  fileout = paste0(config$figures_path,'n_efe_',energy,'_dcpp_seasons_',model,'.png'), 
  lon = data$lon, 
  lat = data$lat, 
  filled.continents = FALSE, 
  filled.oceans = filled.oceans,
  colNA = 'gray',
  titles = paste(letras, ' ',seasons_names),
  brks = brks,
  bar_extra_labels = brks,
  cols = cols,
  units = '%',
  col_inf = "#f5fbff", col_sup = "#1659aa",
  width = 18,
  height = 5,
  triangle_ends = c(T,T)
)

## ACC N_EFE
acc_value_list <- lapply(seasons, function(season) {
  data$metrics[[season]]$n_effective$r
})
acc_sign_list <- lapply(seasons, function(season) {
  data$metrics[[season]]$n_effective$sign
})
brks <- seq(-0.9,0.9,by = 0.1)
special_args <- list()
prop_list <- list()
for (i in 1:length(acc_sign_list)) {
  special_args[[i]] <- list(dots = !acc_sign_list[[i]])  
  prop <- significant_fraction(skill = acc_sign_list[[i]],
                               sign = acc_sign_list[[i]], 
                               lon = data$lon, 
                               lat = data$lat, 
                               positive = TRUE, 
                               mask_ocean = TRUE, 
                               mask_grid = config$grid_25, 
                               londim = 'lon', 
                               latdim = 'lat')
  prop <- prop*100
  prop <- round(prop, 1)
  prop_list <- append(prop_list, prop)
}
titles <- paste(letras, ' ',seasons_names,' (',prop_list,'%)', sep= '')
PlotLayout(
  fun = PlotEquiMap, 
  plot_dims = c('lon', 'lat'), 
  ncol = 5, 
  var = acc_value_list, 
  special_args = special_args,
  fileout = paste0(config$figures_path,'acc_n_efe_',energy,'_seasons_',model,'.png'), 
  lon = data$lon, 
  lat = data$lat, 
  filled.continents = FALSE, 
  filled.oceans = filled.oceans,
  colNA = 'gray',
  titles = titles,
  brks = brks,
  bar_extra_labels = brks,
  width = 18,
  height = 5,
  triangle_ends = c(T,T)
)



}