setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
source('/esarchive/scratch/smoreno/gitlab/capacity_factors/simpl_wind_capacity_factors.R')

plot <- 'solar'
model <- 'EC-Earth3-i4'
member <- 'r1i4p1f1'
region <- 'Iberia'

if (plot == 'solar'){
  sun <- readRDS(file = paste0(config$capacity_factors_models,'solar_capacity_factor_',model,'_fyear1_',region,'_',member,'_first.rds'))
  pv_Annual <- drop(sun$pv_dcpp_Annual)
  pv_Annual <- MeanDims(data = pv_Annual, dims = 'sdate')
  blues <- colorRampPalette(c("#add8e6", "#00008b"))
  blues_soft <- colorRampPalette(c("#e0f3ff", "#78bced", "#1e6ec8"))
  PlotEquiMap(var = pv_Annual, lon = sun$lon, lat = sun$lat, fileout = paste0(config$figures_path,'pv_solar_dcpp_Annual_',model,'_',member,'.png'), filled.continents = FALSE,
              filled.oceans = FALSE, brks = seq(10,22,by = 2), cols = blues_soft(6), toptitle = paste0('Solar Capacity factor - Annual - ', model, ' (',member,')'),
              title_scale = 0.7, width  = 8, height = 8, intxlon = 10, intylat = 10, units = c('%'), triangle_ends = c(T,T), col_inf = "#f5fbff", col_sup = "#1659aa")
  n_efe_Annual <- drop(sun$n_effective_dcpp_Annual)
  n_efe_Annual <- MeanDims(data = n_efe_Annual, dims = 'sdate')
  
  PlotEquiMap(var = n_efe_Annual, lon = sun$lon, lat = sun$lat, fileout = paste0(config$figures_path,'n_efe_solar_Annual_',model,'_',member,'.png'), filled.continents = FALSE,
              filled.oceans = FALSE, brks = seq(85,200,10), cols = blues_soft(length(seq(85,200,10))-1), toptitle = paste0('Number of effective days (rsds > 208 W/m²) - Annual - ', model, ' (',member,')'),
              title_scale = 0.5, width  = 8, height = 8, intxlon = 10, intylat = 10, triangle_ends = c(T,T), col_inf = "#f5fbff", col_sup = "#1659aa")
} else if (plot == 'wind'){
  wind <- readRDS(file = paste0(config$capacity_factors_models,'wind_capacity_factor_',model,'_fyear1_',member,'_',region,'_IEC_class_I.rds'))
  cf_Annual <- drop(wind$cf_dcpp_Annual)
  cf_Annual <- MeanDims(data = cf_Annual, dims = 'sdate')
  blues_soft <- colorRampPalette(c("#e0f3ff", "#78bced", "#1e6ec8"))
  PlotEquiMap(var = cf_Annual*100, lon = wind$lon, lat = wind$lat, fileout = paste0(config$figures_path,'CF_wind_dcpp_Annual_',model,'_',member,'.png'), filled.continents = FALSE,
              filled.oceans = FALSE, brks = seq(20,65,by = 5), cols = blues_soft(length(seq(20,65,5))-1), toptitle = paste0('Wind Capacity factor - Annual - ', model, ' (',member,')'),
              title_scale = 0.7, width  = 8, height = 8, intxlon = 10, intylat = 10, units = c('%'), triangle_ends = c(T,T), col_inf = "#f5fbff", col_sup = "#1659aa", colNA = 'gray')
  sub_Annual <- drop(wind$subdays_dcpp_Annual)
  sub_Annual <- MeanDims(data = sub_Annual, dims = 'sdate')
  sub_Annual[sub_Annual == 0] <- NA
  PlotEquiMap(var = sub_Annual, lon = wind$lon, lat = wind$lat, fileout = paste0(config$figures_path,'n_efe_wind_dcpp_Annual_',model,'_',member,'.png'), filled.continents = FALSE,
              filled.oceans = FALSE, brks = seq(1200,1400,by = 20), cols = blues_soft(length(seq(1200,1400,20))-1), toptitle = paste0('Amount of 6 hourly data with wind production - Annual - ', model, ' (',member,')'),
              title_scale = 0.6, width  = 8, height = 8, intxlon = 10, intylat = 10, triangle_ends = c(T,T), col_inf = "#f5fbff", col_sup = "#1659aa", colNA = 'gray')
}

