setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
library(PCICt)

args <- commandArgs(trailingOnly = TRUE)
model <- args[1]
region <- 'Iberia'
fperiods <- '1'
models <- config$models[1:2]
# config$ncores <- 1
model <- 'BCC-CSM2-MR'

if (identical(fperiods,'all')){
  fperiods <- config$fperiods
} else {
  aux <- unlist(strsplit(x = fperiods, split = ','))
  fperiods <- list()
  for (i in 1:length(aux)){
    auxi <- unlist(strsplit(x = aux[i], split = '-'))
    fperiods[[i]] <- seq(from = auxi[1], to = auxi[length(auxi)], by = 1)
  }
}
fyears <- fperiods
fyears_string <- paste0('fyear',fyears)

members_map <- list(
  'BCC-CSM2-MR' = paste0('r', 1:8, 'i1p1f1'),
  'CanESM5' = paste0('r', 1:20, 'i1p2f1'),
  'CMCC-CM2-SR5' = paste0('r', 1:10, 'i1p1f1'),
  'EC-Earth3-i1' = paste0('r', 1:10, 'i1p1f1'),
  'EC-Earth3-i2' = paste0('r', 6:10, 'i2p1f1'),
  'EC-Earth3-i4' = paste0('r', 1:10, 'i4p1f1'),
  'HadGEM3-GC3.1-MM' = paste0('r', 1:10, 'i1p1f2'),
  'IPSL-CM6A-LR' = paste0('r', 1:10, 'i1p1f1'),
  'MIROC6' = paste0('r', 1:10, 'i1p1f1'),
  'MPI-ESM1.2-HR' = paste0('r', 1:10, 'i1p1f1'),
  'MRI-ESM2-0' = paste0('r', 1:10, 'i1p1f1'),
  'NorCPM1-i1' = paste0('r', 1:10, 'i1p1f1'),
  'NorCPM1-i2' = paste0('r', 1:10, 'i2p1f1')
)
for (model in models){
  print(model)
members <- members_map[[model]]

for (member in members){
  capacity_aux <- readRDS(file = paste0(config$capacity_factors_models,'solar_capacity_factor_',model,'_',fyears_string,'_',region,'_',member,'_first.rds'))
  capacity_aux$pv_dcpp <- capacity_aux$pv_dcpp
  if (member == members[1]){
    capacity_dcpp <- array(data = NA, dim = c(dim(capacity_aux$pv_dcpp), member = config$members[[model]]))
  }
  capacity_dcpp[,,,,which(members == member)] <- capacity_aux$pv_dcpp
}

if (model == models[1]){
  lon <- capacity_aux$lon
  lat <- capacity_aux$lat
  dates <- capacity_aux$dates
  dates_mer <- as.PCICt(x = .POSIXct(xx = sapply(s2dv::Reorder(data = dates, order = c('time','sdate')),I), tz = attr(dates,'tzone')), cal = 'proleptic_gregorian')
  capacity_obs <- capacity_aux$pv_obs
  capacity_obs <- CSTools::MergeDims(data = capacity_obs, merge_dims = c('time','sdate'), rename_dim = 'time', na.rm = FALSE)
  capacity_obs <- s2dv::Reorder(data = capacity_obs, order = c('lon','lat','time'))
}

capacity_dcpp <- CSTools::MergeDims(data = capacity_dcpp, merge_dims = c('time','sdate'), rename_dim = 'time', na.rm = FALSE)
capacity_dcpp <- s2dv::Reorder(data = capacity_dcpp, order = c('lon','lat','time','member'))

months <- format(dates_mer, "%m")
stations <- list(
  DJF = c("12", "01", "02"),
  MAM = c("03", "04", "05"),
  JJA = c("06", "07", "08"),
  SON = c("09", "10", "11"),
  Annual = c('01','02','03','04','05','06','07','08','09','10','11','12')
)
station_results <- list()
for (station in names(stations)) {
  mask <- months %in% stations[[station]]
  capacity_dcpp_station <- capacity_dcpp[, , mask,]
  time_dim_dcpp <- dim(capacity_dcpp_station)['time']
  capacity_obs_station <- capacity_obs[, , mask]
  acc <- CorrEno(exp = capacity_dcpp_station, obs = capacity_obs_station, time_dim = 'time', member_dim = 'member', 
                 method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
  rmsss <- RMSSS_edited(exp = capacity_dcpp_station, obs = capacity_obs_station, ref = NULL, ref_value = 0, 
                        test.type = 'greater', alpha = 0.05, time_dim = 'time', member_dim = 'member', ncores = config$ncores)
  capacity_dcpp_wm <- MeanDims(data = capacity_dcpp_station, dims = 'member')
  if (time_dim_dcpp != dim(capacity_dcpp_station)['time']){
    print(dim(capacity_dcpp_station)['time'])
  }
  if (model == models[1]){
    if (station == 'DJF'){
      multimodel_DJF <- array(data = NA, dim = c(dim(capacity_dcpp_wm), model = length(config$models)))
    } else if (station == 'MAM'){
      multimodel_MAM <- array(data = NA, dim = c(dim(capacity_dcpp_wm), model = length(config$models)))
    } else if (station == 'JJA'){
      multimodel_JJA <- array(data = NA, dim = c(dim(capacity_dcpp_wm), model = length(config$models)))
    } else if (station == 'SON'){
      multimodel_SON <- array(data = NA, dim = c(dim(capacity_dcpp_wm), model = length(config$models)))
    } else if (station == 'Annual'){
      multimodel_Annual <- array(data = NA, dim = c(dim(capacity_dcpp_wm), model = length(config$models)))
    }
  }
  if (station == 'DJF'){
    multimodel_DJF[,,,(which(model == model))] <- capacity_dcpp_wm
  } else if (station == 'MAM'){
    multimodel_MAM[,,,(which(model == model))] <- capacity_dcpp_wm
  } else if (station == 'JJA'){
    multimodel_JJA[,,,(which(model == model))] <- capacity_dcpp_wm
  } else if (station == 'SON'){
    multimodel_SON[,,,(which(model == model))] <- capacity_dcpp_wm
  } else if (station == 'Annual'){
    multimodel_Annual[,,,(which(model == model))] <- capacity_dcpp_wm
  }
  mean_station_dcpp <- MeanDims(data = capacity_dcpp_station, dims = 'time', na.rm = TRUE)
  mean_station_obs <- MeanDims(data = capacity_obs_station, dims = 'time', na.rm = TRUE)
  station_results[[station]] <- list(
    mean_station_dcpp = mean_station_dcpp,
    mean_station_obs = mean_station_obs,
    acc_station = acc,
    rmsss_station = rmsss,
    capacity_dcpp_wm = capacity_dcpp_wm,
    capacity_obs_station = capacity_obs_station
  )
}

saveRDS(object = list(capacity_dcpp_Annual = station_results$Annual$mean_station_dcpp,
                      capacity_obs_Annual = station_results$Annual$mean_station_obs,
                      lon = lon,
                      lat = lat,
                      dates_mer = dates_mer,
                      capacity_dcpp_DJF = station_results$DJF$mean_station_dcpp,
                      capacity_dcpp_MAM = station_results$MAM$mean_station_dcpp,
                      capacity_dcpp_JJA = station_results$JJA$mean_station_dcpp,
                      capacity_dcpp_SON = station_results$SON$mean_station_dcpp,
                      capacity_obs_DJF = station_results$DJF$mean_station_obs,
                      capacity_obs_MAM = station_results$MAM$mean_station_obs,
                      capacity_obs_JJA = station_results$JJA$mean_station_obs,
                      capacity_obs_SON = station_results$SON$mean_station_obs,
                      acc_annual = station_results$Annual$acc,
                      acc_DJF = station_results$DJF$acc,
                      acc_MAM = station_results$MAM$acc,
                      acc_JJA = station_results$JJA$acc,
                      acc_SON = station_results$SON$acc,
                      rmsss_Annual = station_results$Annual$rmsss,
                      rmsss_DJF = station_results$DJF$rmsss,
                      rmsss_MAM = station_results$MAM$rmsss,
                      rmsss_JJA = station_results$JJA$rmsss,
                      rmsss_SON = station_results$SON$rmsss,
                      config = config),
        # file = paste0(config$seasons_solar,'seasons_solar_',model,'_',region,'_',fyears_string,'.rds'))
        file = paste0(config$seasons_solar,'solar_seasons_metrics_',model,'_',region,'_',fyears_string,'.rds'))

}

acc_DJF <- CorrEno(exp = multimodel_DJF, obs = station_results$DJF$capacity_obs_station, time_dim = 'time', member_dim = 'model', 
                   method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
acc_MAM <- CorrEno(exp = multimodel_MAM, obs = station_results$MAM$capacity_obs_station, time_dim = 'time', member_dim = 'model', 
                   method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
acc_JJA <- CorrEno(exp = multimodel_JJA, obs = station_results$JJA$capacity_obs_station, time_dim = 'time', member_dim = 'model', 
                   method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
acc_SON <- CorrEno(exp = multimodel_SON, obs = station_results$SON$capacity_obs_station, time_dim = 'time', member_dim = 'model', 
                   method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
acc_Annual <- CorrEno(exp = multimodel_Annual, obs = station_results$Annual$capacity_obs_station, time_dim = 'time', member_dim = 'model', 
                   method = 'pearson', test.type = 'one-sided', alpha = config$alpha, pval = FALSE, ncores = config$ncores)
rmsss_DJF <- RMSSS_edited(exp = multimodel_DJF, obs = station_results$DJF$capacity_obs_station, ref = NULL, ref_value = 0, 
                          test.type = 'greater', alpha = 0.05, time_dim = 'time', member_dim = 'model', ncores = config$ncores)
rmsss_MAM <- RMSSS_edited(exp = multimodel_MAM, obs = station_results$MAM$capacity_obs_station, ref = NULL, ref_value = 0, 
                        test.type = 'greater', alpha = 0.05, time_dim = 'time', member_dim = 'model', ncores = config$ncores)
rmsss_JJA <- RMSSS_edited(exp = multimodel_JJA, obs = station_results$JJA$capacity_obs_station, ref = NULL, ref_value = 0, 
                        test.type = 'greater', alpha = 0.05, time_dim = 'time', member_dim = 'model', ncores = config$ncores)
rmsss_SON <- RMSSS_edited(exp = multimodel_SON, obs = station_results$SON$capacity_obs_station, ref = NULL, ref_value = 0, 
                       test.type = 'greater', alpha = 0.05, time_dim = 'time', member_dim = 'model', ncores = config$ncores)
rmsss_Annual <- RMSSS_edited(exp = multimodel_Annual, obs = station_results$Annual$capacity_obs_station, ref = NULL, ref_value = 0, 
                           test.type = 'greater', alpha = 0.05, time_dim = 'time', member_dim = 'model', ncores = config$ncores)


saveRDS(object = list(capacity_dcpp_Annual = station_results$Annual$mean_station_dcpp,
                      capacity_obs_Annual = station_results$Annual$mean_station_obs,
                      lon = lon,
                      lat = lat,
                      dates_mer = dates_mer,
                      capacity_dcpp_DJF = multimodel_DJF,
                      capacity_dcpp_MAM = multimodel_MAM,
                      capacity_dcpp_JJA = multimodel_JJA,
                      capacity_dcpp_SON = multimodel_SON,
                      acc_Annual = acc_Annual,
                      acc_DJF = acc_DJF,
                      acc_MAM = acc_MAM,
                      acc_JJA = acc_JJA,
                      acc_SON = acc_SON,
                      rmsss_Annual = rmsss_Annual,
                      rmsss_DJF = rmsss_DJF,
                      rmsss_MAM = rmsss_MAM,
                      rmsss_JJA = rmsss_JJA,
                      rmsss_SON = rmsss_SON,
                      config = config),
        file = paste0(config$seasons_solar,'solar_seasons_metrics_Multi-model-mean','_',region,'_',fyears_string,'.rds'))

print('FINISHED!')

