
library(CSTools)
library(ClimProjDiags)
library(s2dv)
library(qmap)
library(CSIndicators)

simpl_wind_capacity_factors <- function(obs_uas, obs_vas, uas, vas, era5_wind, mask_ori, gwa_spd, agregation, dates_obs, dates_dcpp, fyears, IEC_class, model, machine, config, ncores = 1){

  if (machine != 'nord4'){
    if(!is.null(attr(obs_uas,"NotFoundFiles"))){
      stop(paste0("Missing files:",
                  attr(obs_uas, "NotFoundFiles")[is.na(attr(obs_uas, "Files"))]))
    }
    if(!is.null(attr(obs_vas,"NotFoundFiles"))){
      stop(paste0("Missing files:",
                  attr(obs_vas, "NotFoundFiles")[is.na(attr(obs_vas, "Files"))]))
    }
    if(!is.null(attr(uas,"NotFoundFiles"))){
      stop(paste0("Missing files:",
                  attr(uas, "NotFoundFiles")[is.na(attr(uas, "Files"))]))
    }
    if(!is.null(attr(vas,"NotFoundFiles"))){
      stop(paste0("Missing files:",
                  attr(vas, "NotFoundFiles")[is.na(attr(vas, "Files"))]))
    }
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/make_eval_train_dexes_mine.R')
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/GetProbs_2.R')
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/add_window_dim.R')
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/Quantile_Mapping_fake.R')
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/WindCapacityFactor_duplicate.R')
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/wind2power_duplicate.R')
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/wind2CF_duplicate.R')
    source('/gpfs/scratch/bsc32/bsc32693/scratch/gitlab/capacity_factors/read_pc_duplicate.R')
  } else {
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/make_eval_train_dexes_mine.R')
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/GetProbs_2.R')
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/add_window_dim.R')
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/Quantile_Mapping_fake.R')
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/WindCapacityFactor_duplicate.R')
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/wind2power_duplicate.R')
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/wind2CF_duplicate.R')
    source('/esarchive/scratch/smoreno/gitlab/capacity_factors/read_pc_duplicate.R')
    
  }

  output <- list()
  obs_uas <- drop(obs_uas)
  obs_vas <- drop(obs_vas)
  obs_uas <- s2dv::Reorder(data = obs_uas, order = c('year','time'))
  obs_vas <- s2dv::Reorder(data = obs_vas, order = c('year','time'))
  # Remove 29 Feb and NA from observations
  indices_remove <- apply(dates_obs, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[1]])
  }
  
  obs_uas <- multiApply::Apply(data = obs_uas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  obs_vas <- multiApply::Apply(data = obs_vas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  sfc_wind_obs <- sqrt(obs_uas^2 + obs_vas^2)
  sfc_wind_obs <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'time', indices = keep_indices, drop = FALSE)
  dates_obs <- ClimProjDiags::Subset(x = dates_obs, along = 'time', indices = keep_indices, drop = FALSE)
  rm(obs_uas, obs_vas, keep_indices)
  gc()
  
  uas <- drop(uas)
  vas <- drop(vas)
  sfc_wind_dcpp <- sqrt(uas^2 + vas^2)
  rm(uas,vas)
  gc()
  
  # Convert the sfc winds in winds in 100 m
  multiplication <- ifelse(mask_ori == 0, 1.39, 1.29)
  multiplication <- array(multiplication, dim = dim(sfc_wind_obs))
  sfc_wind_obs <- sfc_wind_obs * multiplication
  sfc_wind_dcpp <- sfc_wind_dcpp * multiplication
  rm(mask_ori,multiplication)
  gc()
  
  # Correct obs data with GWA dataset
  era5_wind <- s2dv::MeanDims(data = era5_wind, dims = c('year','month'))
  ratio <- gwa_spd/era5_wind
  ratio_expanded <- array(ratio, dim = dim(sfc_wind_obs))
  sfc_wind_obs <- sfc_wind_obs * ratio_expanded
  names(dim(sfc_wind_obs))[1] <- 'sdate'
  rm(ratio_expanded,gwa_spd,era5_wind)
  gc()

  if (model == 'IPSL-CM6A-LR'){
    dates_first_year_au <- dates_dcpp[format(dates_dcpp, "%Y") == config$sdates[2]]
    indices_keep_au <- which(format(dates_first_year_au, "%H") %in% c("00", "06", "12", "18"))
    sfc_wind_dcpp <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'time', indices = indices_keep_au, drop = FALSE)
    dcpp_expanded <- abind::abind(sfc_wind_dcpp, sfc_wind_dcpp[,,,1459, drop=FALSE], along = 4)
    names(dim(dcpp_expanded)) <- names(dim(sfc_wind_dcpp))
    sfc_wind_dcpp <- dcpp_expanded
    dates_dcpp <- ClimProjDiags::Subset(x = dates_dcpp, along = 'time', indices = indices_keep_au, drop = FALSE)
  }
  
  sdate_dim <- dim(sfc_wind_obs)['sdate']
  cross <- make_eval_train_dexes_mine(eval.method = 'leave-one-out', amt.points = sdate_dim,
                                      amt.points_cor = NULL, k = 1, tail.out = FALSE)
  dates_first_year_au <- dates_dcpp[1,]
  wind_dcpp_ev_corrected <- array(NA, dim = dim(sfc_wind_dcpp)[2])
  wind_dcpp_ev_corrected <- s2dv::InsertDim(data = wind_dcpp_ev_corrected, posdim = 1, lendim = 1, name = 'sdate')
  wind_dcpp_ev_corrected <- s2dv::Reorder(data = wind_dcpp_ev_corrected, order = c('sdate','time')) #,'member'))
  years_len <- length(config$sdates)-1
  wind_dcpp_tr_corrected <- array(NA, dim = dim(sfc_wind_dcpp[1:years_len,]))
  months <- format(dates_dcpp, "%m")
  seasons <- list(
    DJF = c("12", "01", "02"),
    MAM = c("03", "04", "05"),
    JJA = c("06", "07", "08"),
    SON = c("09", "10", "11"),
    Annual = c('01','02','03','04','05','06','07','08','09','10','11','12')
  )
  seasons_masks <- lapply(seasons, function(seasons_months) {
    format(dates_first_year_au, "%m") %in% seasons_months
  })

  output$cf_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$cf_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$cf_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$cf_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$cf_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$cf_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$cf_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$cf_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$cf_obs_SON <- array(NA, dim = c(sdate_dim))
  output$cf_obs_Annual <- array(NA, dim = c(sdate_dim))
  output$cf_probs_DJF <- array(NA, dim = c(bin = 3, sdate_dim))
  output$cf_probs_MAM <- array(NA, dim = c(bin = 3, sdate_dim))
  output$cf_probs_JJA <- array(NA, dim = c(bin = 3, sdate_dim))
  output$cf_probs_SON <- array(NA, dim = c(bin = 3, sdate_dim))
  output$cf_probs_Annual <- array(NA, dim = c(bin = 3, sdate_dim))
  output$subdays_dcpp_DJF <- array(NA, dim = c(sdate_dim))
  output$subdays_dcpp_MAM <- array(NA, dim = c(sdate_dim))
  output$subdays_dcpp_JJA <- array(NA, dim = c(sdate_dim))
  output$subdays_dcpp_SON <- array(NA, dim = c(sdate_dim))
  output$subdays_dcpp_Annual <- array(NA, dim = c(sdate_dim))
  output$subdays_obs_DJF <- array(NA, dim = c(sdate_dim))
  output$subdays_obs_MAM <- array(NA, dim = c(sdate_dim))
  output$subdays_obs_JJA <- array(NA, dim = c(sdate_dim))
  output$subdays_obs_SON <- array(NA, dim = c(sdate_dim))
  output$subdays_obs_Annual <- array(NA, dim = c(sdate_dim))

  
  for (t in 1:sdate_dim){
    print(t)
    wind_obs_tr <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    wind_dcpp_tr <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    wind_dcpp_ev <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    wind_obs_ev <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    for (hour in c("00", "06", "12", "18")){
      indices_hour <- which(format(dates_first_year_au, "%H") == hour)
      wind_dcpp_ev_hour <- ClimProjDiags::Subset(x = wind_dcpp_ev, along = 'time', indices = indices_hour, drop = FALSE)
      wind_dcpp_tr_hour <- ClimProjDiags::Subset(x = wind_dcpp_tr, along = 'time', indices = indices_hour, drop = FALSE)
      wind_obs_tr_hour <- ClimProjDiags::Subset(x = wind_obs_tr, along = 'time', indices = indices_hour, drop = FALSE)
      wind_dcpp_tr_hour_wi <- add_window_dim(wind_dcpp_tr_hour)
      wind_obs_tr_hour_wi <- add_window_dim(wind_obs_tr_hour)
      
      wind_dcpp_tr_hour <- Quantile_Mapping_fake(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                                 exp_cor = wind_dcpp_tr_hour,
                                                 na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                                 window_dim = 'window',
                                                 ncores = config$ncores, method = 'QUANT', qstep = 0.01)
      
      wind_dcpp_ev_hour <- Quantile_Mapping_fake(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                                 exp_cor = wind_dcpp_ev_hour, 
                                                 na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                                 window_dim = 'window',
                                                 ncores = config$ncores, method = 'QUANT', qstep = 0.01)
      wind_dcpp_ev_corrected[,indices_hour] <- wind_dcpp_ev_hour
      wind_dcpp_tr_corrected[,indices_hour] <- wind_dcpp_tr_hour
    }
    wind_dcpp_tr <- wind_dcpp_tr_corrected
    
      if (t == 1){
        # cf_dcpp_DJF <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_dcpp_MAM <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_dcpp_JJA <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_dcpp_SON <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_dcpp_Annual <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_obs_DJF <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_obs_MAM <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_obs_JJA <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_obs_SON <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_obs_Annual <- array(NA, dim = dim(wind_dcpp_ev_corrected))
        # cf_probs_DJF <- array(NA, dim = c(bin = 3, dim(wind_dcpp_ev_corrected)))
        # cf_probs_MAM <- array(NA, dim = c(bin = 3, dim(wind_dcpp_ev_corrected)))
        # cf_probs_JJA <- array(NA, dim = c(bin = 3, dim(wind_dcpp_ev_corrected)))
        # cf_probs_SON <- array(NA, dim = c(bin = 3, dim(wind_dcpp_ev_corrected)))
        # cf_probs_Annual <- array(NA, dim = c(bin = 3, dim(wind_dcpp_ev_corrected)))
        # subdays_dcpp_DJF <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_obs_DJF <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_dcpp_MAM <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_obs_MAM <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_dcpp_JJA <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_obs_JJA <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_dcpp_SON <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_obs_SON <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_dcpp_Annual <- array(NA, dim = dim(cf_dcpp_DJF))
        # subdays_obs_Annual <- array(NA, dim = dim(cf_dcpp_DJF))
      }
      
       for (season in names(seasons)) {
        seasons_mask <- seasons_masks[[season]]
        dates_season <- dates_dcpp[,seasons_mask]
        wind_dcpp_ev_season <- wind_dcpp_ev_corrected[,seasons_mask]
        wind_dcpp_ev_season <- s2dv::InsertDim(data = wind_dcpp_ev_season, posdim = 1, lendim = 1, name = 'sdate')
        names(dim(wind_dcpp_ev_season))[2] <- 'time'
        wind_dcpp_tr_season <- wind_dcpp_tr[,seasons_mask]
        names(dim(wind_dcpp_tr_season))[2] <- 'time'
        wind_obs_ev_season <- wind_obs_ev[,seasons_mask]
        wind_obs_ev_season <- s2dv::InsertDim(data = wind_obs_ev_season, posdim = 1, lendim = 1, name = 'sdate')
        names(dim(wind_obs_ev_season))[2] <- 'time'
        cf_dcpp_ev <- WindCapacityFactor_duplicate(wind = wind_dcpp_ev_season, IEC_class = IEC_class, dates = dates_dcpp, ncores = config$ncores)
        cf_dcpp_tr <- WindCapacityFactor_duplicate(wind = wind_dcpp_tr_season, IEC_class = IEC_class, dates = dates_dcpp, ncores = config$ncores)
        cf_obs <- WindCapacityFactor_duplicate(wind = wind_obs_ev_season, IEC_class = IEC_class, dates = dates_dcpp, ncores = config$ncores)
        cf_dcpp_ev <- s2dv::MeanDims(data = cf_dcpp_ev, dims = 'time', na.rm = TRUE)
        cf_dcpp_tr <- s2dv::MeanDims(data = cf_dcpp_tr, dims = 'time', na.rm = TRUE)
        cf_obs <- s2dv::MeanDims(data = cf_obs, dims = 'time', na.rm = TRUE)
        lims_cf_tr <- multiApply::Apply(cf_dcpp_tr, target_dims = c('sdate'),
                                        fun = function(x) {quantile(as.vector(x), c(1/3, 2/3), na.rm = TRUE)},
                                        output_dims = 'bin', ncores = config$ncores)$output1
        cf_probs_ev <- GetProbs_2(cf_dcpp_ev, time_dim = 'sdate', 
                                  prob_thresholds = NULL,
                                  bin_dim_abs = 'bin', 
                                  indices_for_quantiles = NULL,
                                  memb_dim = NULL, abs_thresholds = lims_cf_tr,
                                  ncores = config$ncores)
        cf_probs_ev <- s2dv::Reorder(data = cf_probs_ev, order = c('bin','sdate'))
        valid_subdays_dcpp <- apply(wind_dcpp_ev_season, 1, function(x) sum(x > config$IEC_speeds[[IEC_class]]$cut_in_speed & x < config$IEC_speeds[[IEC_class]]$cut_out_speed, na.rm = TRUE))
        valid_subdays_obs <- apply(wind_obs_ev_season, 1, function(x) sum(x > config$IEC_speeds[[IEC_class]]$cut_in_speed & x < config$IEC_speeds[[IEC_class]]$cut_out_speed, na.rm = TRUE))
        # dates_first_y_season <- dates_season[1,]
        # days <- format(dates_first_y_season, "%Y-%m-%d")
        # unique_days <- unique(days)
        # wind_dcpp_ev_daily <- array(NA, dim = c(sdate = 1, time = length(unique_days)))
        # wind_obs_ev_daily <- array(NA, dim = c(sdate = 1, time = length(unique_days)))
        # for (i in seq_along(unique_days)) {
        #   daily_mask <- days == unique_days[i]
        #   wind_dcpp_ev_daily[, i] <- apply(wind_dcpp_ev_season[, daily_mask, drop = FALSE], 1, mean) 
        #   wind_obs_ev_daily[, i] <- apply(wind_obs_ev_season[, daily_mask, drop = FALSE], 1, mean) 
        # }
        # if (season == 'DJF'){
        #   output$cf_dcpp_DJF[[t]] <- cf_dcpp_ev
        #   output$cf_obs_DJF[[t]] <- cf_obs
        #   output$cf_probs_DJF[,t] <- cf_probs_ev
        #   output$subdays_dcpp_DJF[[t]] <- valid_subdays_dcpp
        #   output$subdays_obs_DJF[[t]] <- valid_subdays_obs
        # } else if (season == 'MAM'){
        #   output$cf_dcpp_MAM[[t]] <- cf_dcpp_ev
        #   output$cf_obs_MAM[[t]] <- cf_obs
        #   output$cf_probs_MAM[,t] <- cf_probs_ev
        #   output$subdays_dcpp_MAM[[t]] <- valid_subdays_dcpp
        #   output$subdays_obs_MAM[[t]] <- valid_subdays_obs
        # } else if (season == 'JJA'){
        #   output$cf_dcpp_JJA[[t]] <- cf_dcpp_ev
        #   output$cf_obs_JJA[[t]] <- cf_obs
        #   output$cf_probs_JJA[,t] <- cf_probs_ev
        #   output$subdays_dcpp_JJA[[t]] <- valid_subdays_dcpp
        #   output$subdays_obs_JJA[[t]] <- valid_subdays_obs
        # } else if (season == 'SON'){
        #   output$cf_dcpp_SON[[t]] <- cf_dcpp_ev
        #   output$cf_obs_SON[[t]] <- cf_obs
        #   output$cf_probs_SON[,t] <- cf_probs_ev
        #   output$subdays_dcpp_SON[[t]] <- valid_subdays_dcpp
        #   output$subdays_obs_SON[[t]] <- valid_subdays_obs
        # } else if (season == 'Annual'){
        #   output$cf_dcpp_Annual[[t]] <- cf_dcpp_ev
        #   output$cf_obs_Annual[[t]] <- cf_obs
        #   output$cf_probs_Annual[,t] <- cf_probs_ev
        #   output$subdays_dcpp_Annual[[t]] <- valid_subdays_dcpp
        #   output$subdays_obs_Annual[[t]] <- valid_subdays_obs
        # }
        output[[paste0('cf_dcpp_', season)]][[t]] <- cf_dcpp_ev
        output[[paste0('cf_obs_', season)]][[t]] <- cf_obs
        output[[paste0('cf_probs_', season)]][, t] <- cf_probs_ev
        output[[paste0('subdays_dcpp_', season)]][[t]] <- valid_subdays_dcpp
        output[[paste0('subdays_obs_', season)]][[t]] <- valid_subdays_obs
        
     }

  }
  return(output)
}
