library(startR)
path_models <- ('/esarchive/exp/CMIP6/dcppA-hindcast/')
startR_calls_daily <- function(variables, dataset, fyears, lon_min, lon_max, lat_min, lat_max, config, retrieve, members, interpolation = FALSE, grid = config$grid_25, sdates = config$sdates){
  print(dataset)
  ##############################################################################################################################
  #################################################### OBSERVATIONS ############################################################
  ##############################################################################################################################
  
  if (identical(dataset,'BEST')){
    
    data <- startR::Start(dat = '/esarchive/obs/berkeleyearth/berkeleyearth/daily/$var$/$var$_$year$$month$.nc', var = variables, 
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          year = paste0(config$years), month = 'all', time = 'all',
                          time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = FALSE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
    calendar <- 'gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K')
    
  } else if (identical(dataset,'ERA5land')){
    
    if (machine_index == 'am' | machine_index == 'gs' | machine_index == 'da') {
      if (identical(variables, 'tasmax') | identical(variables, 'tasmin')) {
        path <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/recon/era5land/daily/$var$/$var$_$year$$month$.nc'
      } else if (identical(variables,'pr')){
        path <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/recon/era5land/daily_mean/$var$_f1h/$var$_$year$$month$.nc'
        variables <- 'prlr'
      } else if (identical(variables,'tas')){
        path <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/recon/era5land/daily_mean/$var$_f1h/$var$_$year$$month$.nc'
      }
    } else {
      if (identical(variables, 'tasmax') | identical(variables, 'tasmin')) {
        path <- '/esarchive/recon/ecmwf/era5land/daily/$var$/$var$_$year$$month$.nc'
      } else if (identical(variables,'pr')){
        path <- '/esarchive/recon/ecmwf/era5land/daily_mean/$var$_f1h/$var$_$year$$month$.nc'
        variables <- 'prlr'
      } else if (identical(variables,'tas')){
        path <- '/esarchive/recon/ecmwf/era5land/daily_mean/$var$_f1h/$var$_$year$$month$.nc'
      }
    } 
    
    config$years <- 1960:2020

    data <- startR::Start(dat = path, var = variables, 
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          year = paste0(config$years), month = 'all', time = 'all',
                          time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
    calendar <- 'proleptic_gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'm s-1')
    
    } else if (identical(dataset,'ERA5')){
      path <- '/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'

      data <- startR::Start(dat = path, var = variables, 
                            lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                            year = paste0(config$years), month = 'all', time = 'all',
                            time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = FALSE,
                            synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                            return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                            lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                            num_procs = 1, retrieve = retrieve)
    
    calendar <- 'proleptic_gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'm s-1')
    
    ##############################################################################################################################
    ################################################# DECADAL PREDICTIONS ########################################################
    ##############################################################################################################################
    
  } else if (identical(dataset,'EC-Earth3')){
    
    calendar <- 'proleptic_gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Nov')
    # exp_list <- list(); for (sdt in config$sdates){if (sdt <= 2020){exp_list[[as.character(sdt)]] <- 'dcppA-hindcast'} else {exp_list[[as.character(sdt)]] <- 'dcppB-forecast'}}
    
    data <- startR::Start(dat = '/esarchive/exp/ecearth/a3w5/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/day/$var$/gr/v20210910/$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$fyear$.nc',
                          var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste(config$sdates), time = startR::indices(times), fyear = 'all', member = config$etccdi_members[[dataset]],
                          fyear_depends = 'sdate',
                          time_across = 'fyear', merge_across_dims = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
  } else if (identical(dataset,'HadGEM3-GC3.1-MM')){
    
    calendar <- '360_day'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Nov') ## It is initialised in January, but files contain data for calendar years (Jan to Dec)

    data <- startR::Start(dat = paste0(path_models,'HadGEM3-GC31-MM/DCPP/MOHC/HadGEM3-GC31-MM/dcppA-hindcast/$member$/day/$var$/gn/v20200417/$var$__HadGEM3-GC31-MM_dcppA-hindcast_s$sdate$-$member$_gn_$fyear$0101-$fyear$1230.nc'), var = variables,
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste0(config$sdates), time = indices(times), member = members,
                          fyear = 'all', fyear_depends = 'sdate', 
                          time_across = 'fyear', merge_across_dims = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)


  } else if (identical(dataset,'BCC-CSM2-MR')){

    if (is.null(members)){
          if (identical(config$members[[dataset]],8)){members <- paste0('r',1:8,'i1p1f1')} else {stop('Check the members')}
    }

    calendar <- '365_day'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Jan')

    if (identical(variables,'psl') | identical(variables,'tasmin') | identical(variables,'tasmax') | identical(variables,'pr') | identical(variables,'rsds')){
      version <- 'v20200114'
    } else if (identical(variables,'tas')){
      version <- 'v20200408'
    } 
    
    if (interpolation == TRUE){
      data <- startR::Start(dat = paste0(path_models,'BCC-CSM2-MR/DCPP/BCC/BCC-CSM2-MR/dcppA-hindcast/$member$/day/$var$/gn/',version,'/$var$_day_BCC-CSM2-MR_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'),
                            var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                            sdate = paste(config$sdates+1), time = startR::indices(times), member = members,
                            time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                            synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                            return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                            lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                            transform = CDORemapper,
                            transform_extra_cells = 2,
                            transform_params = list(grid = config$grid_25,
                                                    method = 'conservative'),
                            transform_vars = c('lat', 'lon'),
                            num_procs = config$ncores, retrieve = retrieve)
    } else {
      data <- startR::Start(dat = paste0(path_models,'BCC-CSM2-MR/DCPP/BCC/BCC-CSM2-MR/dcppA-hindcast/$member$/day/$var$/gn/',version,'/$var$_day_BCC-CSM2-MR_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'),
                            var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                            sdate = paste(config$sdates+1), time = startR::indices(times), member = members,
                            time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                            synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                            return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                            lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                            num_procs = config$ncores, retrieve = retrieve)
    }
    
  } else if (identical(dataset,'CanESM5')){
    
    if (is.null(members)){
      if (identical(config$members[[dataset]],20)){members <- paste0('r',1:20,'i1p2f1')} else {stop('Check the members')}
    }
    calendar <- '365_day'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Jan')

    data <- startR::Start(dat = paste0(path_models,'CanESM5/DCPP/CCCma/CanESM5/dcppA-hindcast/$member$/day/$var$/gn/v20190429/$var$_day_CanESM5_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'),
                          var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste(config$sdates), time = startR::indices(times), member = members,
                          aux = 'all', aux_depends = 'sdate',
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
  } else if (identical(dataset,'CMCC-CM2-SR5')){
    
    if (is.null(members)){
      if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i1p1f1')} else {stop('Check the members')}
    }
    
    calendar <- '365_day'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Nov')
    
    if (variables %in% c('tasmin','tasmax','pr')){
      version <- 'v20210909'
      day <- 'day'
    } else if (identical(variables,'psl') | (identical(variables,'tas'))){
      version <- 'v20230130'
      day <- 'day'
    } else if (identical(variables,'zg500')){
      version <- 'v20240614'
      day <- 'AERday'
    } else if (identical(variables,'rsds')){
      version <- 'v20240614'
      day <- 'day'
    }
    data <- startR::Start(dat = paste0(path_models,'CMCC-CM2-SR5/DCPP/CMCC/CMCC-CM2-SR5/dcppA-hindcast/$member$/',day,'/$var$/gn/',version,'/$var$_',day,'_CMCC-CM2-SR5_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'),
                          var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste(config$sdates), time = indices(times), member = members,
                          aux = 'all', aux_depends = 'sdate', 
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)

    
  } else if (identical(dataset,'EC-Earth3-i1')){
    
    if (is.null(members)){
      if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i1p1f1')} else {stop('Check the members')}
    }
    calendar <- 'proleptic_gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Nov')
    
    if (variables != 'zg500' & variables != 'rsds'){
      path <- ('/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/')
    } else {
      path <- ('/esarchive/exp/CMIP6/dcppA-hindcast/EC-Earth3/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/')
    }

    data <- startR::Start(dat = paste0(path,'$member$/day/$var$/gr/v20190713/$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$fyear$.nc'),
                          var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste(config$sdates), time = indices(times), fyear = 'all', member = members,
                          fyear_depends = 'sdate', time_across = 'fyear', merge_across_dims = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
  } else if (identical(dataset,'EC-Earth3-i2')){

      if (is.null(members)){
        if (identical(config$members[[dataset]],5)){members <- paste0('r',6:10,'i2p1f1')} else {stop('Check the members')}
      }
    # 
    calendar <- 'proleptic_gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Jan') ## it is actually initialised in November, but the files begin in Jan

    if (identical(variables,'tasmax') | identical(variables,'tasmin')){
      version <- 'v20200730'
    } else if (identical(variables, 'pr')){
      version <- 'v20200508'
    } else if (identical(variables, 'psl')){
      version <- 'v20220322'
    } else if (identical(variables, 'tas')){
      version <- 'v20200731'
    } else if (identical(variables,'rsds')){
      version <- 'v20240620'
    }
    path <- paste0(path_models,'EC-Earth3/')
    
    data <- startR::Start(dat = paste0(path,'DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/day/$var$/gr/',version,'/$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$fyear$0101-$fyear$1231.nc'),
                          var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste(config$sdates), time = indices(times), fyear = 'all', member = members,
                          # version = list(tasmin = 'v20200730', tasmax = 'v20200730', pr = 'v20200508'), version_depends = 'var',
                          fyear_depends = 'sdate', time_across = 'fyear', merge_across_dims = TRUE,
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)

    
  } else if (identical(dataset,'EC-Earth3-i4')){

    calendar <- 'proleptic_gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Nov')
    exp_list <- list(); for (sdt in sdates){if (sdt <= 2020){exp_list[[as.character(sdt)]] <- 'dcppA-hindcast'} else {exp_list[[as.character(sdt)]] <- 'dcppB-forecast'}}
    initmonth_list <- list(); for (sdt in sdates){if (sdt <= 2023){initmonth_list[[as.character(sdt)]] <- ''} else {initmonth_list[[as.character(sdt)]] <- '1101'}}
    data <- startR::Start(dat = '/esarchive/exp/ecearth/cmip6-$exp$/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/$exp$/$member$/day/$var$/gr/v20210910/$var$_day_EC-Earth3_$exp$_s$sdate$$initmonth$-$member$_gr_$fyear$.nc',
                          var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste(sdates), time = startR::indices(times), fyear = 'all', member = members, 
                          exp = exp_list, exp_depends = 'sdate', initmonth = initmonth_list, 
                          fyear_depends = 'sdate', exp_depends = 'fyear', fyear_depends = 'exp', initmonth_depends = 'sdate',
                          time_across = 'fyear', merge_across_dims = TRUE,
                          transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = grid, method = 'conservative', force_remap = TRUE), transform_vars = c('lat','lon'),
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)

    # version <- 'v20210910'
    # path <- ('/esarchive/exp/ecearth/a3w5/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/')
    # 
    # 
    # data <- startR::Start(dat = paste0(path,'$member$/day/$var$/gr/',version,'/$var$_day_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$fyear$.nc'),
    #                       var = variables, lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
    #                       sdate = paste(config$sdates), time = indices(times), fyear = 'all', member = members,
    #                       fyear_depends = 'sdate', time_across = 'fyear', merge_across_dims = TRUE,
    #                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
    #                       return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
    #                       lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
    #                       transform = CDORemapper,
    #                       transform_extra_cells = 2,
    #                       transform_params = list(grid = config$grid_25,
    #                                               method = 'conservative'),
    #                       transform_vars = c('lat', 'lon'),
    #                       num_procs = 1, retrieve = retrieve)
    
    
  } else if (identical(dataset,'IPSL-CM6A-LR')){
    if (is.null(members)){
      if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i1p1f1')} else {stop('Check the members')}
    }
    calendar <- 'gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Jan')
    if (fyears >= 5){times <- times + 1} ## the first day is not load due to leap year


    if (identical(variables,'psl') | identical(variables,'tas')){
      data <- startR::Start(dat = paste0(path_models,'IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/day/$var$/gr/$version$/$var$_day_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'), var = variables,
                            lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                            sdate = paste0(config$sdates), time = indices(times), member = members,
                            version = 'all', version_depends = 'var', version_depends = 'aux',
                            aux = 'all', aux_depends = 'sdate', aux_depends = 'version',
                            return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                            lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                            num_procs = 1, retrieve = retrieve)
    } else if (variables == 'rsds'){
      data <- startR::Start(dat = paste0(path_models,'IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/day/$var$/gr/v20200108/$var$_day_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'), var = variables,
                            lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                            sdate = paste0(config$sdates), time = indices(times), member = members,
                            aux = 'all', aux_depends = 'sdate', aux_depends = 'version',
                            return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                            lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                            num_procs = 1, retrieve = retrieve)
    }

  } else if (identical(dataset,'MIROC6')){
    
    if (is.null(members)){
      if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i1p1f1')} else {stop('Check the members')}
    }

    calendar <- 'standard'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    if (identical(variables,'tasmax') | identical(variables,'tasmin') | identical(variables,'tas')){
      version <- 'v20200416'
    } else if (identical(variables,'pr') | identical(variables,'psl') | identical(variables,'zg500') | identical(variables,'rsds')){
      version <- 'v20191217'
    }
    if (variables == 'zg500'){
      month <- 'Jan'
      path <- paste0(path_models,'MIROC6/DCPP/MIROC/MIROC6/dcppA-hindcast/$member$/day/$var$/gn/',version,'/$var$_day_MIROC6_dcppA-hindcast_s$sdate$-$member$_gn_$aux$0101-$aux$1231.nc')
    } else {
      month <- 'Nov'
      path <- paste0(path_models,'MIROC6/DCPP/MIROC/MIROC6/dcppA-hindcast/$member$/day/$var$/gn/',version,'/$var$_day_MIROC6_dcppA-hindcast_s$sdate$-$member$_gn_$sdate$1101-$aux$1231.nc')
    }
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = month) #jan para zg500
    if (fyears > 4){times <- times + 1}

          data <- startR::Start(dat = path, var = variables,
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste0(config$sdates), time = indices(times), member = members,
                          # version = list(tasmin = 'v20200416', tasmax = 'v20200416', pr = 'v20191217'), version_depends = 'var',
                          aux = 'all', aux_depends = 'sdate',merge_across_dims = TRUE,time_across = 'aux',
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    # }

    
  } else if (identical(dataset,'MPI-ESM1.2-HR')){
    
    # if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i1p1f1')} else {stop('Check the members')}
    calendar <- 'standard'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Nov')
    if (fyears > 4){times <- times + 1}
    
    day <- 'day'
    if (variables == "tasmin") {
      version <- "v20200101"
    } else if (variables == "pr" | variables == 'rsds') {
      version <- "v20210128"
    } else if (variables == "psl") {
      version <- "v20210130"
    } else if (variables == "tas") {
      version <- "v20200320"
    } else if (variables == 'zg500') {
      version <- 'v20210128'
      day <- 'AERday'
    }

    data <- startR::Start(dat = paste0(path_models,'MPI-ESM1-2-HR/DCPP/MPI-M/MPI-ESM1-2-HR/dcppA-hindcast/$member$/',day,'/$var$/gn/',version,'/$var$_',day,'_MPI-ESM1-2-HR_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'), var = variables,
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste0(config$sdates), time = indices(times), member = members,
                          # version = list(tasmin = 'v20200101', tasmax = 'v20200101', pr = 'v20210128'), version_depends = 'var', 
                          aux = 'all', aux_depends = 'sdate', 
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
  } else if (identical(dataset,'MRI-ESM2-0')){
    
    if (is.null(members)){
    if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i1p1f1')} else {stop('Check the members')}
    }
    calendar <- 'proleptic_gregorian'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Nov')
    if (fyears > 4){times <- times + 1}
    
    data <- startR::Start(dat = paste0(path_models,'MRI-ESM2-0/DCPP/MRI/MRI-ESM2-0/dcppA-hindcast/$member$/day/$var$/gn/v20210122/$var$_day_MRI-ESM2-0_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'), var = variables,
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste0(config$sdates), time = indices(times), member = members,
                          aux = 'all', aux_depends = 'sdate',
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
  } else if (identical(dataset,'NorCPM1-i1')){
    
    # if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i1p1f1')} else {stop('Check the members')}
    calendar <- 'noleap'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Oct')
    times <- times - 14 ## this model is initialised in Oct 15th
    
    if (identical(variables,'psl') | identical(variables,'tasmin') | identical(variables,'tasmax') | identical(variables,'pr') | identical(variables,'rsds')){
      version <- 'v20191005'
    } else if (identical(variables, 'tas')){
      version <- 'v20200320'
    } 
    
    data <- startR::Start(dat = paste0(path_models,'NorCPM1/DCPP/NCC/NorCPM1/dcppA-hindcast/$member$/day/$var$/gn/',version,'/$var$_day_NorCPM1_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'), var = variables,
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste0(config$sdates), time = indices(times), member = members, 
                          aux = 'all', aux_depends = 'sdate',
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
  } else if (identical(dataset,'NorCPM1-i2')){
    
    if (identical(config$members[[dataset]],10)){members <- paste0('r',1:10,'i2p1f1')} else {stop('Check the members')}
    calendar <- 'noleap'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    times <- get_fdays(fyears = fyears, calendar = calendar, month_init = 'Oct')
    times <- times - 14 ## this model is initialised in Oct 15th
    
    day <- 'day'
    if (identical(variables,'psl') | identical(variables,'tasmin') | identical(variables,'tasmax') | identical(variables,'pr') | identical(variables,'rsds')){
      version <- 'v20191005'
    } else if (identical(variables, 'tas')){
      version <- 'v20200101'
    } else if (identical(variables,'zg500')){
      version <- 'v20191005'
    }
    
    data <- startR::Start(dat = paste0(path_models,'NorCPM1/DCPP/NCC/NorCPM1/dcppA-hindcast/$member$/',day,'/$var$/gn/',version,'/$var$_',day,'_NorCPM1_dcppA-hindcast_s$sdate$-$member$_gn_$aux$.nc'), var = variables,
                          lon = values(list(lon_min,lon_max)), lat = values(list(lat_min,lat_max)),
                          sdate = paste0(config$sdates), time = indices(times), member = members,
                          aux = 'all', aux_depends = 'sdate',
                          return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)
    
  } else {stop('Incorrect dataset')}
  
  lon <- attr(data,'Variables')$common[['lon']]
  lat <- attr(data,'Variables')$common[['lat']]
  dates <- attr(data,'Variables')$common$time
  stopifnot(!is.null(lat))
  stopifnot(!is.null(lon))
  stopifnot(!is.null(dates))
  if(!is.null(attr(data,"NotFoundFiles"))){
    stop(paste0("Missing files:", 
                attr(data, "NotFoundFiles")[is.na(attr(data, "Files"))]))
  }
  
  stopifnot(is.null(attr(data,'NotFoundFiles')))

  return(list(data = data, 
              lat = lat, 
              lon = lon,
              dates = dates,
              calendar = calendar, 
              units = units))
}


