
library(startR)
path_models <- ('/esarchive/exp/CMIP6/dcppA-hindcast/')

startR_calls_interpolated_wind <- function(model,fy, region, config, retrieve, member){
  print(model)
  ##############################################################################################################################
  #################################################### OBSERVATIONS ############################################################
  ##############################################################################################################################
  if (model == 'EC-Earth3-i2'){
      path_models <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/EC-Earth3/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/6hrPlevPt/sfcWind/gr/v20210309/sfcWind_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
      calendar <- '6hr'
      times <- get_fdays(fyears = fy, calendar = calendar, month_init = 'Nov')
      sfc_wind_dcpp <- startR::Start(dat = path_models, var = 'sfcWind',
                                lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                sdate = paste(config$sdates), member = member, time = startR::indices(times), #time_across = 'fyear',
                                time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                                synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                                lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                transform = CDORemapper,
                                transform_extra_cells = 2,
                                transform_params = list(grid = config$grid_25,
                                                        method = 'conservative'),
                                transform_vars = c('lat', 'lon'),
                                num_procs = 1, retrieve = retrieve)
      
      lon <- attr(sfc_wind_dcpp,'Variables')$common[['lon']]
      lat <- attr(sfc_wind_dcpp,'Variables')$common[['lat']]
      dates <- attr(sfc_wind_dcpp,'Variables')$common$time
      stopifnot(!is.null(lat))
      stopifnot(!is.null(lon))
      stopifnot(!is.null(dates))
      if(!is.null(attr(sfc_wind_dcpp,"NotFoundFiles"))){
        stop(paste0("Missing files:", 
                    attr(sfc_wind_dcpp, "NotFoundFiles")[is.na(attr(sfc_wind_dcpp, "Files"))]))
      }
      
      stopifnot(is.null(attr(sfc_wind_dcpp,'NotFoundFiles')))
      
      return(list(uas = uas,
                  vas = vas,
                  lat = lat, 
                  lon = lon,
                  dates = dates,
                  calendar = calendar, 
                  units = units))
      
  } else {
    if (identical(model, 'EC-Earth3-i1')) {
      calendar <- '6hr'
      units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
      times <- get_fdays(fyears = fy, calendar = calendar, month_init = 'Nov')
      path_models <- paste0('/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20190713/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
    } else if (identical(model, 'EC-Earth3-i4')) {
      calendar <- '6hr'
      month_init <- 'Nov'
      # path_models <- '/esarchive/exp/ecearth/cmip6-$exp$/original_files/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/$exp$/$member$/6hrPlevPt/$var$/gr/v20210910/$var$_6hrPlevPt_EC-Earth3_$exp$_s$sdate$$initmonth$-$member$_gr_$aux$.nc'
      path_models <- paste0('/esarchive/scratch/smoreno/capacity_outputs/temporal_files/$var$/EC-Earth3-i4/$member$/final_',region,'_$var$_6hrPlevPt_EC-Earth3_$exp$_s$sdate$$initmonth$-$member$_gr_$aux$.nc')
      exp_list <- list(); for (sdt in config$sdates){if (sdt <= 2020){exp_list[[as.character(sdt)]] <- 'dcppA-hindcast'} else {exp_list[[as.character(sdt)]] <- 'dcppB-forecast'}}
      initmonth_list <- list(); for (sdt in config$sdates){if (sdt <= 2023){initmonth_list[[as.character(sdt)]] <- ''} else {initmonth_list[[as.character(sdt)]] <- '1101'}}
    } else if (identical(model, 'IPSL-CM6A-LR')) {
      calendar <- '3hr'
      units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
      month_init <- 'Jan'
      if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
        path_models <- paste0('/gpfs/projects/bsc32/esarchive_cache/exp/CMIP6/dcppA-hindcast/IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/3hr/$var$/gr/v20200504/$var$_3hr_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
      } else {
        path_models <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/3hr/$var$/gr/v20200504/$var$_3hr_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
      }
    }
    
    times <- get_fdays(fyears = fy, calendar = calendar, month_init = month_init)
    uas <- startR::Start(
      dat = list(list(path = path_models,pattern_dims = list(exp = "exp",member = "member",var = "var",sdate = "sdate",aux = "aux"))),
      var = 'uas',lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
      lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),sdate = paste(config$sdates),time = indices(times),
      member = member,exp = exp_list,aux = "all",initmonth = initmonth_list, initmonth_depends = 'sdate',aux_depends = "sdate",
      exp_depends = "sdate",time_across = "aux",merge_across_dims = TRUE,synonims = list(lat = c('lat', 'latitude'), lon = c('lon', 'longitude'), time = c('time')),
      return_vars = list(lat = NULL, lon = NULL, time = c('sdate')),lon_var = 'lon',lat_var = 'lat',lat_reorder = Sort(decreasing = FALSE),
      num_procs = 1,retrieve = retrieve
    )

    vas <- startR::Start(
      dat = list(list(path = path_models,pattern_dims = list(exp = "exp",member = "member",var = "var",sdate = "sdate",aux = "aux"))),
      var = 'vas', lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
      lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)), sdate = paste(config$sdates),time = indices(times),
      member = member,exp = exp_list,aux = "all",initmonth = initmonth_list, initmonth_depends = 'sdate',aux_depends = "sdate",
      exp_depends = "sdate",time_across = "aux",merge_across_dims = TRUE,synonims = list(lat = c('lat', 'latitude'), lon = c('lon', 'longitude'), time = c('time')),
      return_vars = list(lat = NULL, lon = NULL, time = c('sdate')),lon_var = 'lon',lat_var = 'lat',lat_reorder = Sort(decreasing = FALSE),
      num_procs = 1, retrieve = retrieve
    )
    
    calendar <- '365_day'
    units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
    lon <- attr(uas,'Variables')$common[['lon']]
    lat <- attr(uas,'Variables')$common[['lat']]
    dates <- attr(uas,'Variables')$common$time
    stopifnot(!is.null(lat))
    stopifnot(!is.null(lon))
    stopifnot(!is.null(dates))
    if(!is.null(attr(uas,"NotFoundFiles"))){
      stop(paste0("Missing files:", 
                  attr(uas, "NotFoundFiles")[is.na(attr(uas, "Files"))]))
    }
    if(!is.null(attr(vas,"NotFoundFiles"))){
      stop(paste0("Missing files:", 
                  attr(vas, "NotFoundFiles")[is.na(attr(vas, "Files"))]))
    }
    
    stopifnot(is.null(attr(uas,'NotFoundFiles')))
    stopifnot(is.null(attr(vas,'NotFoundFiles')))
    
    return(list(uas = uas,
                vas = vas,
                lat = lat, 
                lon = lon,
                dates = dates,
                calendar = calendar, 
                units = units))
    
  }
  
}


