library(startR)
path_models <- ('/esarchive/scratch/smoreno/capacity_outputs/temporal_files')
startR_wind_temporal_files <- function(model,fy, config, retrieve, member, region){
  print(model)
  ##############################################################################################################################
  #################################################### OBSERVATIONS ############################################################
  ##############################################################################################################################
  if (model == 'EC-Earth3-i2'){
    if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
      path_models <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/scratch/exp/CMIP6/dcppA-hindcast/EC-Earth3-i4/DCPP/EC-Earth-Consortium/EC-Earth3-i4/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20210910/uas_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'
    } else {
      path_models <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/EC-Earth3/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/6hrPlevPt/sfcWind/gr/v20210309/sfcWind_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
    }
      calendar <- '6hr'
      times <- get_fdays(fyears = fy, calendar = calendar, month_init = 'Nov')
      sfc_wind_dcpp <- startR::Start(dat = path_models, var = 'sfcWind',
                                lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                                lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                                sdate = paste(config$sdates), member = member, time = startR::indices(times), #time_across = 'fyear',
                                time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                                synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                                return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                                lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                                transform = CDORemapper,
                                transform_extra_cells = 2,
                                transform_params = list(grid = config$grid_25,
                                                        method = 'conservative'),
                                transform_vars = c('lat', 'lon'),
                                num_procs = 1, retrieve = retrieve)
      
      lon <- attr(sfc_wind_dcpp,'Variables')$common[['lon']]
      lat <- attr(sfc_wind_dcpp,'Variables')$common[['lat']]
      dates <- attr(sfc_wind_dcpp,'Variables')$common$time
      stopifnot(!is.null(lat))
      stopifnot(!is.null(lon))
      stopifnot(!is.null(dates))
      if(!is.null(attr(sfc_wind_dcpp,"NotFoundFiles"))){
        stop(paste0("Missing files:", 
                    attr(sfc_wind_dcpp, "NotFoundFiles")[is.na(attr(sfc_wind_dcpp, "Files"))]))
      }
      
      stopifnot(is.null(attr(sfc_wind_dcpp,'NotFoundFiles')))
      
      return(list(uas = uas,
                  vas = vas,
                  lat = lat, 
                  lon = lon,
                  dates = dates,
                  calendar = calendar, 
                  units = units))
      
  } else {
    if (identical(model, 'EC-Earth3-i1')) {
      calendar <- '6hr'
      units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
      times <- get_fdays(fyears = fy, calendar = calendar, month_init = 'Nov')
      if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
        path_models <- paste0('/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/scratch/exp/CMIP6/dcppA-hindcast/EC-Earth3-i1/DCPP/EC-Earth-Consortium/EC-Earth3-i1/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20190713/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
      } else {
        path_models <- paste0('/esarchive/exp/ecearth/a1ua/cmorfiles/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$member$/6hrPlevPt/$var$/gr/v20190713/$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
      }
    } else if (identical(model, 'EC-Earth3-i4')) {
      calendar <- '6hr'
      units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
      month_init <- 'Nov'
    } else if (identical(model, 'IPSL-CM6A-LR')) {
      calendar <- '3hr'
      units <- list('tasmin' = 'K', 'tasmax' = 'K', 'pr' = 'kg m-2 s-1')
      month_init <- 'Jan'
      if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
        path_models <- paste0('/gpfs/projects/bsc32/esarchive_cache/exp/CMIP6/dcppA-hindcast/IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/3hr/$var$/gr/v20200504/$var$_3hr_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
      } else {
        path_models <- paste0('/esarchive/exp/CMIP6/dcppA-hindcast/IPSL-CM6A-LR/DCPP/IPSL/IPSL-CM6A-LR/dcppA-hindcast/$member$/3hr/$var$/gr/v20200504/$var$_3hr_IPSL-CM6A-LR_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc')
      }
    }
    
    #### REHACER get_fdays PARA QUE TENGA EN CUENTA FY !!!
    times <- get_fdays(fyears = 1, calendar = calendar, month_init = month_init)
    uas <- startR::Start(dat = paste0(path,'/$var$/',model,'/$member$/final_',region,'$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'),
                         lon = 'all',lat = 'all',
                         sdate = paste(config$sdates), time = indices(times), member = member, #time_across = 'fyear',
                         time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                         lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                         num_procs = 1, retrieve = retrieve)
    
    uas <- startR::Start(dat = paste0(path,'/$var$/',model,'/$member$/final_',region,'$var$_6hrPlevPt_EC-Earth3_dcppA-hindcast_s$sdate$-$member$_gr_$aux$.nc'),
                         lon = 'all',lat = 'all',
                         sdate = paste(config$sdates), member = member, time = indices(times),
                         time_across = 'aux', merge_across_dims = TRUE, aux = 'all', aux_depends = 'sdate',
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = NULL, lon = NULL, time = c('sdate')), lon_var = 'lon', lat_var = 'lat',
                         lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                         num_procs = 1, retrieve = retrieve)

    calendar <- '365_day'
    
    lon <- attr(uas,'Variables')$common[['lon']]
    lat <- attr(uas,'Variables')$common[['lat']]
    dates <- attr(uas,'Variables')$common$time
    stopifnot(!is.null(lat))
    stopifnot(!is.null(lon))
    stopifnot(!is.null(dates))
    if(!is.null(attr(uas,"NotFoundFiles"))){
      stop(paste0("Missing files:", 
                  attr(uas, "NotFoundFiles")[is.na(attr(uas, "Files"))]))
    }
    if(!is.null(attr(vas,"NotFoundFiles"))){
      stop(paste0("Missing files:", 
                  attr(vas, "NotFoundFiles")[is.na(attr(vas, "Files"))]))
    }
    
    stopifnot(is.null(attr(uas,'NotFoundFiles')))
    stopifnot(is.null(attr(vas,'NotFoundFiles')))
    
    return(list(uas = uas,
                vas = vas,
                lat = lat, 
                lon = lon,
                dates = dates,
                calendar = calendar, 
                units = units))
    
  }
  
}


