setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')
source('means_test.R')
region <- 'Gal'
retrieve <- FALSE
config$years <- 1990:1992
time_dim <- 'sdate'
sys_info <- Sys.info()['nodename']
machine_index <- substr(sys_info, 1, 2)
# if (machine == 'nord4'){
path <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
path_mask <- '/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/'
era5_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'  
gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/temporal_files/aux/GWA_$var$m_int_Europe.nc"


total_hours <- 8928
desired_hours <- c(0, 6, 12, 18)
selected_indices <- NULL
for (hour in desired_hours) {
  selected_indices <- c(selected_indices, seq(hour + 1, total_hours, by = 24))
}
obs_uas <- startR::Start(dat = path, var = 'uas',
                         lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                         lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                         year = paste0(config$years), month = 'all', time = selected_indices,
                         time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                         lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                         num_procs = 1, retrieve = retrieve)

obs_vas <- startR::Start(dat = path, var = 'vas',
                         lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                         lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                         year = paste0(config$years), month = 'all', time = selected_indices,
                         time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                         synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                         return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                         lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                         num_procs = 1, retrieve = retrieve)

dates_obs <- attr(obs_vas,"Variables")$common$time
lon <- attr(obs_vas,'Variables')$common[['lon']]
lat <- attr(obs_vas,'Variables')$common[['lat']]

era5_wind <- startR::Start(dat = era5_path, var = 'sfcWind',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', time = 'all',
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = retrieve)


gwa <- startR::Start(dat = gwa100_file, var = 'WSPD100',
                     lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                     lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                     synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                     return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                     lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                     num_procs = 1, retrieve = retrieve)

mask_ori <- startR::Start(dat = paste0(path_mask,'$var$.nc'), var = 'sftof',
                          lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                          lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                          synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                          return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                          lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                          num_procs = 1, retrieve = retrieve)

dcpp <- startR_calls_interpolated_wind(model = model,fy = fy, config = config, region = region, member = member,
                                       retrieve = retrieve)
uas <- dcpp$uas
vas <- dcpp$vas

chars <- apply(attr(obs_uas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
  paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(x, 36, nchar(x)))})
attr(obs_uas,"ExpectedFiles") <- chars
obs_uas[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(obs_uas[2], 36, nchar(obs_uas[2])))
chars <- apply(attr(obs_vas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
  paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(x, 36, nchar(x)))})
attr(obs_vas,"ExpectedFiles") <- chars
obs_vas[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5/1hourly", substr(obs_vas[2], 36, nchar(obs_vas[2])))
chars <- apply(attr(uas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
  paste0("/gpfs/scratch/bsc32/bsc32693/scratch", substr(x, 27, nchar(x)))})
attr(uas,"ExpectedFiles") <- chars
uas[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch", substr(uas[2], 38, nchar(uas[2])))
chars <- apply(attr(vas,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
  paste0("/gpfs/scratch/bsc32/bsc32693/scratch", substr(x, 27, nchar(x)))})
attr(vas,"ExpectedFiles") <- chars
vas[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch", substr(vas[2], 38, nchar(vas[2])))
chars <- apply(attr(era5_wind,"ExpectedFiles"), c(1,2,3,4), FUN = function(x) {
  paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/era5", substr(x, 28, nchar(x)))})
attr(era5_wind,"ExpectedFiles") <- chars
era5_wind[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/recon/", substr(era5_wind[2], 24, nchar(era5_wind[2])))
chars <- apply(attr(gwa,"ExpectedFiles"), c(1), FUN = function(x) {
  paste0("/gpfs/scratch/bsc32/bsc32693/scratch/", substr(x, 28, nchar(x)))})
attr(gwa,"ExpectedFiles") <- chars
gwa[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/", substr(gwa[2], 28, nchar(gwa[2])))
chars <- apply(attr(mask_ori,"ExpectedFiles"), c(1), FUN = function(x) {
  paste0("/gpfs/scratch/bsc32/bsc32693/scratch/capacity_outputs/temporal_files/aux/", substr(x, 55, nchar(x)))})
attr(mask_ori,"ExpectedFiles") <- chars
mask_ori[2] <- paste0("/gpfs/scratch/bsc32/bsc32693/scratch/capacity_outputs/temporal_files/aux/", substr(mask_ori[2], 24, nchar(mask_ori[2])))

target_dims <-c('year','month')
output_dims <- list(data = 'year')


if (isFALSE(retrieve)) {
  ETW
  step <- startR::Step(fun = means_test,
                       target_dims = target_dims, 
                       output_dims = output_dims)
  
  wf <- startR::AddStep(inputs = list(data = era5_wind),
                        config = config,
                        ncores = config$ncores,
                        step_fun = step)
  
  chunks <- list(lat = 3, lon = 3) 
  job_wallclock <- '01:00:00'
  config$ncores <- 1

  output <- Compute(workflow = wf,
                    chunks = chunks,
                    threads_load = 1,
                    threads_compute = config$ncores,
                    cluster = list(queue_host = 'amd',
                                   r_module = "R/4.1.2-foss-2019b",
                                   temp_dir = '/gpfs/scratch/bsc32/bsc032693/startR_hpc/',
                                   CDO_module = "CDO/1.9.8-foss-2019b",
                                   ecflow_module = 'ecFlow',
                                   cores_per_job = 7,
                                   job_wallclock = job_wallclock,
                                   max_jobs = 10,
                                   extra_queue_params = list('#SBATCH -A bsc32', '#SBATCH -q bsc_es'),
                                   bidirectional = FALSE,
                                   polling_period = 10),
                    ecflow_suite_dir = '/home/Earth/smoreno/Desktop/startR_local/',
                    wait = TRUE)
  
  
  
} else {stop('retrieve must be TRUE or FALSE')}


saveRDS(object = list(obs = output$obs,
                      lon = lon,
                      lat = lat,
                      config = config),
        file = paste0(config$objetos_aux,'test_amd.rds'))

print(paste0('Finished in ',round(x = (proc.time() - time_starting_exec)[3]/60, digits = 0),' min.'))

