
library(CSTools)
library(ClimProjDiags)
library(s2dv)
library(qmap)
library(CSIndicators)

valid_wind_days <- function(obs_uas, obs_vas, uas, vas, era5_wind, mask_ori, gwa_spd, agregation, dates_obs, dates_dcpp, fyears, IEC_class, model, config, ncores = 1){
  
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/make_eval_train_dexes_mine.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/GetProbs_2.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/add_window_dim.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/Quantile_Mapping_fake.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/WindCapacityFactor_duplicate.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/wind2power_duplicate.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/wind2CF_duplicate.R')
  source('/esarchive/scratch/smoreno/gitlab/capacity_factors/read_pc_duplicate.R')

  output <- list()
  obs_uas <- drop(obs_uas)
  obs_vas <- drop(obs_vas)
  obs_uas <- s2dv::Reorder(data = obs_uas, order = c('year','time'))
  obs_vas <- s2dv::Reorder(data = obs_vas, order = c('year','time'))
  # Remove 29 Feb
  indices_remove <- apply(dates_obs, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[1]])
  }

  obs_uas <- multiApply::Apply(data = obs_uas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  obs_vas <- multiApply::Apply(data = obs_vas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  sfc_wind_obs <- sqrt(obs_uas^2 + obs_vas^2)
  sfc_wind_obs <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'time', indices = keep_indices, drop = FALSE)
  dates_obs <- ClimProjDiags::Subset(x = dates_obs, along = 'time', indices = keep_indices, drop = FALSE)
  era5_wind <- s2dv::MeanDims(data = era5_wind, dims = c('year','month'))
  ratio <- gwa_spd/era5_wind
  ratio_expanded <- array(ratio, dim = dim(sfc_wind_obs))
  sfc_wind_obs <- sfc_wind_obs * ratio_expanded
  names(dim(sfc_wind_obs))[1] <- 'sdate'
  
  uas <- drop(uas)
  vas <- drop(vas)
  sfc_wind_dcpp <- sqrt(uas^2 + uas^2)
  if (model == 'IPSL-CM6A-LR'){
    dates_first_year_au <- dates_dcpp[format(dates_dcpp, "%Y") == config$sdates[2]]
    indices_keep_au <- which(format(dates_first_year_au, "%H") %in% c("00", "06", "12", "18"))
    sfc_wind_dcpp <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'time', indices = indices_keep_au, drop = FALSE)
    dcpp_expanded <- abind::abind(sfc_wind_dcpp, sfc_wind_dcpp[,,,1459, drop=FALSE], along = 4)
    names(dim(dcpp_expanded)) <- names(dim(sfc_wind_dcpp))
    sfc_wind_dcpp <- dcpp_expanded
    dates_dcpp <- ClimProjDiags::Subset(x = dates_dcpp, along = 'time', indices = indices_keep_au, drop = FALSE)
  }
  
  sdate_dim <- dim(sfc_wind_obs)['sdate']
  cross <- make_eval_train_dexes_mine(eval.method = 'leave-one-out', amt.points = sdate_dim,
                                  amt.points_cor = NULL, k = 1, tail.out = FALSE)
  dates_first_year_au <- dates_dcpp[1,]
  wind_dcpp_ev_corrected <- array(NA, dim = dim(sfc_wind_dcpp)[2])
  wind_dcpp_ev_corrected <- s2dv::InsertDim(data = wind_dcpp_ev_corrected, posdim = 1, lendim = 1, name = 'sdate')
  wind_dcpp_ev_corrected <- s2dv::Reorder(data = wind_dcpp_ev_corrected, order = c('sdate','time')) #,'member'))
  years_len <- length(config$sdates)-1
  wind_dcpp_tr_corrected <- array(NA, dim = dim(sfc_wind_dcpp[1:years_len,]))
  months <- format(dates_dcpp, "%m")
  seasons <- list(
    DJF = c("12", "01", "02"),
    MAM = c("03", "04", "05"),
    JJA = c("06", "07", "08"),
    SON = c("09", "10", "11"),
    Annual = c('01','02','03','04','05','06','07','08','09','10','11','12')
  )
  seasons_masks <- lapply(seasons, function(seasons_months) {
    format(dates_first_year_au, "%m") %in% seasons_months
  })
  multiplication <- ifelse(mask_ori == 0, 1.39, 1.29)
  
  if (IEC_class == 'I'){
    cut_in_speed <- 2
    cut_out_speed <- 25
  } else if (IEC_class == 'II'){
    cut_in_speed <- 4
    cut_out_speed <- 25
  } else if (IEC_class == 'III'){
    cut_in_speed <- 3
    cut_out_speed <- 20
  } else if (IEC_class == 'I/II'){
    cut_in_speed <- 4
    cut_out_speed <- 25
  } else if (IEC_class == 'II/III'){
    cut_in_speed <- 3
    cut_out_speed <- 20
  }

  for (t in 1:sdate_dim){
    print(t)
    wind_obs_tr <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    wind_dcpp_tr <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$train.dexes, drop = 'selected')
    wind_dcpp_ev <- ClimProjDiags::Subset(x = sfc_wind_dcpp, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    wind_obs_ev <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdate', indices = cross[[t]]$eval.dexes, drop = FALSE)
    for (hour in c("00", "06", "12", "18")){
      indices_hour <- which(format(dates_first_year_au, "%H") == hour)
      wind_dcpp_ev_hour <- ClimProjDiags::Subset(x = wind_dcpp_ev, along = 'time', indices = indices_hour, drop = FALSE)
      wind_dcpp_tr_hour <- ClimProjDiags::Subset(x = wind_dcpp_tr, along = 'time', indices = indices_hour, drop = FALSE)
      wind_obs_tr_hour <- ClimProjDiags::Subset(x = wind_obs_tr, along = 'time', indices = indices_hour, drop = FALSE)
      wind_dcpp_tr_hour_wi <- add_window_dim(wind_dcpp_tr_hour)
      wind_obs_tr_hour_wi <- add_window_dim(wind_obs_tr_hour)
      wind_dcpp_tr_hour <- Quantile_Mapping_fake(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                           exp_cor = wind_dcpp_tr_hour,
                                           na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                           window_dim = 'window',
                                           ncores = config$ncores, method = 'QUANT', qstep = 0.01)

      wind_dcpp_ev_hour <- Quantile_Mapping_fake(exp = wind_dcpp_tr_hour_wi, obs = wind_obs_tr_hour_wi, 
                                           exp_cor = wind_dcpp_ev_hour, 
                                           na.rm = TRUE,  memb_dim = NULL, sdate_dim = 'sdate', 
                                           window_dim = 'window',
                                           ncores = config$ncores, method = 'QUANT', qstep = 0.01)
      wind_dcpp_ev_corrected[,indices_hour] <- wind_dcpp_ev_hour
      wind_dcpp_tr_corrected[,indices_hour] <- wind_dcpp_tr_hour
    }
    multiplication_exp <- array(multiplication, dim = dim(wind_obs_tr))
    wind_obs_tr <- wind_obs_tr * multiplication_exp
    wind_dcpp_tr <- wind_dcpp_tr_corrected * multiplication_exp
    multiplication_exp <- array(multiplication, dim = dim(wind_obs_ev))
    wind_obs_ev <- wind_obs_ev * multiplication_exp
    wind_dcpp_ev_corrected <- wind_dcpp_ev_corrected * multiplication_exp
    
    if (agregation == 'monthly'){
      
    } else if (agregation == 'seasons'){
      if (t == 1){
        days_dcpp_DJF <- array(NA, dim = c(sdate = 1,  n_days_au[['DJF']]))
        days_obs_DJF <- array(NA, dim = c(sdate = 1,  n_days_au[['DJF']]))
        days_dcpp_MAM <- array(NA, dim = c(sdate = 1,  n_days_au[['MAM']]))
        days_obs_MAM <- array(NA, dim = c(sdate = 1,  n_days_au[['MAM']]))
        days_dcpp_JJA <- array(NA, dim = c(sdate = 1,  n_days_au[['JJA']]))
        days_obs_JJA <- array(NA, dim = c(sdate = 1,  n_days_au[['JJA']]))
        days_dcpp_SON <- array(NA, dim = c(sdate = 1,  n_days_au[['SON']]))
        days_obs_SON <- array(NA, dim = c(sdate = 1,  n_days_au[['SON']]))
        days_dcpp_Annual <- array(NA, dim = c(sdate = 1,  n_days_au[['Annual']]))
        days_obs_Annual <- array(NA, dim = c(sdate = 1,  n_days_au[['Annual']]))
      }
      
      for (season in names(seasons)) {
        seasons_mask <- seasons_masks[[season]]
        dates_season <- dates_dcpp[,seasons_mask]
        wind_dcpp_ev_season <- wind_dcpp_ev_corrected[,seasons_mask]
        wind_dcpp_ev_season <- s2dv::InsertDim(data = wind_dcpp_ev_season, posdim = 1, lendim = 1, name = 'sdate')
        names(dim(wind_dcpp_ev_season))[2] <- 'time'
        wind_dcpp_tr_season <- wind_dcpp_tr[,seasons_mask]
        names(dim(wind_dcpp_tr_season))[2] <- 'time'
        wind_obs_ev_season <- wind_obs_ev[,seasons_mask]
        wind_obs_ev_season <- s2dv::InsertDim(data = wind_obs_ev_season, posdim = 1, lendim = 1, name = 'sdate')
        names(dim(wind_obs_ev_season))[2] <- 'time'
        dates_first_y_season <- dates_season[1,]
        days <- format(dates_first_y_season, "%Y-%m-%d")
        unique_days <- unique(days)
        n_days <- length(unique_days)
        valid_days <- array(FALSE, dim = c(time = n_days))
        valid_days_obs <- array(FALSE, dim = c(time = n_days))
        for (i in seq_len(n_days)) {
          idx <- which(days == unique_days[i])  
          day_vals <- wind_dcpp_ev_season[,idx, drop = FALSE]
          day_vals_obs <- wind_obs_ev_season[,idx, drop = FALSE]
          condition <- (day_vals > cut_in_speed) & (day_vals < cut_out_speed)
          condition_obs <- (day_vals_obs > cut_in_speed) & (day_vals_obs < cut_out_speed)
          count <- apply(condition, c(1), sum) 
          count_obs <- apply(condition_obs, c(1), sum) 
          valid_days[i] <- count >= 2
          valid_days_obs[i] <- count_obs >= 2
        }

        if (season == 'DJF'){
          output$days_dcpp_DJF[t,] <- valid_days
          output$days_obs_DJF[t,] <- valid_days_obs
        } else if (season == 'MAM'){
          output$days_dcpp_MAM[t,] <- valid_days
          output$days_obs_MAM[t,] <- valid_days_obs
        } else if (season == 'JJA'){
          output$days_dcpp_JJA[t,] <- valid_days
          output$days_obs_JJA[t,] <- valid_days_obs
        } else if (season == 'SON'){
          output$days_dcpp_SON[t,] <- valid_days
          output$days_obs_SON[t,] <- valid_days_obs
        } else if (season == 'Annual'){
          output$days_dcpp_Annual[t,] <- valid_days
          output$days_obs_Annual[t,] <- valid_days_obs
        }
      }
    } else if (agregation == 'daily'){
    }

  }
  return(output)
}
