# setwd('/esarchive/scratch/smoreno/gitlab/capacity_factors/')
source('config.R')

args <- commandArgs(trailingOnly = TRUE)
model <- args[1]
member <- args[2]
region <- args[3]
loocv <- args[4]
hour_to_dow <- args[5]
fperiods <- '1'
# model <- 'IPSL-CM6A-LR'
region_big <- 'Atlantic'
loocv <- 1
# member <- 'r1i1p1f1'
# region <- 'Cat'
# regions <- list('Atlantic' = list('lon_min' = 0, 'lon_max' = 4, 'lat_min' = 40, 'lat_max' = 45),
#                'Cat' = list('lon_min' = 0, 'lon_max' = 3, 'lat_min' = 40, 'lat_max' = 43))
# config$years <- 1990:1992
# config$sdates <- 1990:1992
if (region == 'Europe'){
  grid <- config$grid_25
} else {
  grid <- config$grid_down
}

if (identical(models,'all')){
  models <- c(config$models,config$multimodels)
} else {
  models <- unlist(strsplit(x = models, split = ','))
}

if (identical(fperiods,'all')){
  fperiods <- config$fperiods
} else {
  aux <- unlist(strsplit(x = fperiods, split = ','))
  fperiods <- list()
  for (i in 1:length(aux)){
    auxi <- unlist(strsplit(x = aux[i], split = '-'))
    fperiods[[i]] <- seq(from = auxi[1], to = auxi[length(auxi)], by = 1)
  }
}

for (fyears in fperiods){
  time_starting_exec <- proc.time()
  
  if (length(fyears) == 1){fyears_string <- paste0('fyear',fyears[1])} else {fyears_string <- paste0('fyears',fyears[1],'-',fyears[length(fyears)])}
  sdates <- config$sdates[1] : (config$years[length(config$years)] - as.integer(fyears[length(fyears)]))
  years <- (sdates[1] + as.integer(fyears[1])) : config$years[length(config$years)]
  
  print(paste0('Starting ',model,' - ',fyears_string))
  
  ################################################################################################
  #################################### Loading the data ##########################################
  ################################################################################################
  
  print('Loading the data...')
  
  time_dim <- 'sdate'
  sys_info <- Sys.info()['nodename']
  machine_index <- substr(sys_info, 1, 2)
  if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
    path <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/recon/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
    path_mask <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/'
    path_era5 <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/recon/era5/daily_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'
  } else {
    path <- '/esarchive/recon/ecmwf/era5/1hourly/$var$-r1440x721cds/$var$_$year$$month$.nc'
    path_mask <- '/esarchive/recon/ecmwf/era5/constant/lsm-r1440x721cds/'
    path_era5 <- '/esarchive/recon/ecmwf/era5/daily_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'
  }
  
  obs_uas <- startR::Start(dat = path, var = 'uas',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', time = 'all',
                           time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = TRUE)
  
  lon_uas <- attr(obs_uas,'Variables')$common[['lon']]
  lat_uas <- attr(obs_uas,'Variables')$common[['lat']]
  dates_uas <- attr(obs_uas,'Variables')$common$time
  
  # Remove 29 Feb
  indices_remove <- apply(dates_uas, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[1]])
  }
  obs_uas <- ClimProjDiags::Subset(x = obs_uas, along = 'time', indices = keep_indices, drop = FALSE)
  dates_uas <- ClimProjDiags::Subset(x = dates_uas, along = 'time', indices = keep_indices, drop = FALSE)
  dates_first_year <- dates_uas[format(dates_uas, "%Y") == config$years[1]]
  indices_keep <- which(format(dates_first_year, "%H") %in% hour_to_dow)
  obs_uas <- ClimProjDiags::Subset(x = obs_uas, along = 'time', indices = indices_keep, drop = FALSE)
  dates_uas <- ClimProjDiags::Subset(x = dates_uas, along = 'time', indices = indices_keep, drop = FALSE)
  
  obs_vas <- startR::Start(dat = path, var = 'vas',
                           lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                           lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                           year = paste0(config$years), month = 'all', time = 'all',
                           time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                           synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                           return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                           lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                           num_procs = 1, retrieve = TRUE)
  
  
  lon_vas <- attr(obs_vas,'Variables')$common[['lon']]
  lat_vas <- attr(obs_vas,'Variables')$common[['lat']]
  dates_vas <- attr(obs_vas,'Variables')$common$time
  # Remove 29 Feb
  indices_remove <- apply(dates_vas, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_vas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_vas)['time']) %in% indices_remove[[1]])
  }
  obs_vas <- ClimProjDiags::Subset(x = obs_vas, along = 'time', indices = keep_indices, drop = FALSE)
  dates_vas <- ClimProjDiags::Subset(x = dates_vas, along = 'time', indices = keep_indices, drop = FALSE)
  dates_first_year <- dates_vas[format(dates_vas, "%Y") == config$years[1]]
  indices_keep <- which(format(dates_first_year, "%H") %in% hour_to_dow)
  obs_vas <- ClimProjDiags::Subset(x = obs_vas, along = 'time', indices = indices_keep, drop = FALSE)
  dates_vas <- ClimProjDiags::Subset(x = dates_vas, along = 'time', indices = indices_keep, drop = FALSE)
  
  obs_lons <- as.vector(lon_uas)
  obs_lats <- as.vector(lat_uas)
  # indices_years <- which(config$years %in% years)
  obs_uas <- multiApply::Apply(data = obs_uas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  # obs_uas <- s2dv::CDORemap(data_array = obs_uas, lons = obs_lons, lats = obs_lats, grid = grid, method = config$interpolation_method, crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
  obs_vas <- multiApply::Apply(data = obs_vas, target_dims = 'year', 
                               fun = function(data){zoo::rollmean(x = as.vector(data), k = length(fyears), fill = NULL, align = 'left')}, 
                               output_dims = 'year', ncores = config$ncores)$output1
  # obs_vas <- s2dv::CDORemap(data_array = obs_vas, lons = obs_lons, lats = obs_lats, grid = grid, method = config$interpolation_method, crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
  # obs_lons <- obs_uas$lons
  # obs_lats <- obs_uas$lats
  # obs_uas <- obs_uas$data_array
  # obs_vas <- obs_vas$data_array
  obs_uas <- drop(obs_uas)
  obs_vas <- drop(obs_vas)
  obs_uas <- Reorder(data = obs_uas, order = c('year','lon','lat','time'))
  obs_vas <- Reorder(data = obs_vas, order = c('year','lon','lat','time'))
  sfc_wind_obs <- sqrt(obs_uas^2 + obs_vas^2)
  rm(obs_vas,obs_uas)
  
  obsL <- startR::Start(dat = path_era5, var = 'psl', 
                        lon = values(list(config$regions[[region_big]]$lon_min,config$regions[[region_big]]$lon_max)), 
                        lat = values(list(config$regions[[region_big]]$lat_min,config$regions[[region_big]]$lat_max)),
                        year = paste0(config$years), month = 'all', time = 'all',
                        time_across = 'month', merge_across_dims = TRUE, merge_across_dims_narm = TRUE,
                        synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                        return_vars = list(lat = NULL, lon = NULL, time = c('month','year')), lon_var = 'lon', lat_var = 'lat',
                        lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                        transform = CDORemapper,
                        transform_extra_cells = 2,
                        transform_params = list(grid = config$grid_up,
                                                method = 'conservative'),
                        transform_vars = c('lat', 'lon'),
                        num_procs = 1, retrieve = TRUE)
  
  obsL_lons <- attr(obsL,'Variables')$common[['lon']]
  obsL_lats <- attr(obsL,'Variables')$common[['lat']]
  dates_obsL <- attr(obsL,'Variables')$common$time
  
  indices_remove <- apply(dates_obsL, 1, function(x) {which(is.na(x) == T)})
  if (config$years[1] %% 4 == 0 && (config$years[1] %% 100 != 0 || config$years[1] %% 400 == 0)){
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[2]])
  } else {
    keep_indices <- !(seq_len(dim(obs_uas)['time']) %in% indices_remove[[1]])
  }
  obsL <- ClimProjDiags::Subset(x = obsL, along = 'time', indices = keep_indices, drop = FALSE)
  dates_obsL <- ClimProjDiags::Subset(x = dates_obsL, along = 'time', indices = keep_indices, drop = FALSE)
  obsL <- drop(obsL)
  obsL_lons <- as.vector(obsL_lons)
  obsL_lats <- as.vector(obsL_lats)
  names(dim(obsL))[3] <- 'sdates'
  
  # Corrección porque las observaciones de ERA5 tienen errores
  if (machine_index == 'gl' | machine_index == 'as' | machine_index == 'gs' | machine_index == 'da'){
    era5_path <- '/gpfs/projects/bsc32/esarchive_cache/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'
    gwa100_file <- '/gpfs/scratch/bsc32/MN4/bsc32/bsc32693/data_amd/scratch/capacity_outputs/GWA_wspd100m_t170grid.nc'
  } else {
    era5_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h-r1440x721cds/$var$_$year$$month$.nc'  
    gwa100_file <- "/esarchive/scratch/smoreno/capacity_outputs/GWA_wspd100m_t170grid.nc"
  }
  era5_wind <- startR::Start(dat = era5_path, var = 'sfcWind',
                             lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                             lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                             year = paste0(config$years), month = 'all', time = 'all',
                             synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                             return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                             lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                             num_procs = 1, retrieve = TRUE)
  lon_era5 <- attr(era5_wind,'Variables')$common[['lon']]
  lat_era5 <- attr(era5_wind,'Variables')$common[['lat']]
  lat_era5 <- as.vector(lat_era5)
  lon_era5 <- as.vector(lon_era5)
  era5_wind <- drop(era5_wind)
  era5_wind <- MeanDims(data=era5_wind, dims = c('year','month'))
  gwa.nc <- nc_open(gwa100_file)
  gwa_spd <- ncvar_get(gwa.nc,"WSPD100")
  names(dim(gwa_spd)) <- c('lon','lat')
  lat <- ncvar_get(gwa.nc,"latitude")
  lon <- ncvar_get(gwa.nc,"longitude")
  era5_wind <- s2dv::CDORemap(data_array = era5_wind, lons = lon_era5, lats = lat_era5, grid = grid, method = config$interpolation_method, 
                              crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
  gwa_spd <- s2dv::CDORemap(data_array = gwa_spd, lons = lon, lats = lat, grid = grid, method = config$interpolation_method, 
                            crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
  ratio <- gwa_spd$data_array/era5_wind$data_array
  ratio_expanded <- array(ratio, dim = dim(sfc_wind_obs))
  sfc_wind_obs <- sfc_wind_obs * ratio_expanded
  
  fy <- 1
  times <- get_fdays(fyears = fy, calendar = calendar, month_init = month_init)
  print(member)
  dcpp_psl <- startR_calls_daily(dataset = model, variables = 'psl', fyears = fy, config = config, lon_min = config$regions[[region_big]]$lon_min, lon_max = config$regions[[region_big]]$lon_max,
                                 lat_min = config$regions[[region_big]]$lat_min, lat_max = config$regions[[region_big]]$lat_max, retrieve = TRUE, members = member)
  lon_dcpp <- dcpp_psl$lon
  lat_dcpp <- dcpp_psl$lat
  aux <- dcpp_psl$data
  aux <- drop(aux)
  dcpp <- Reorder(data = aux, order = c('sdate','lon','lat','time'))
  days <- as.numeric(dim(dcpp)["time"])

  names(dim(sfc_wind_obs))[1] <- 'sdates'
  sdate_dim <- dim(sfc_wind_obs)['sdates']
  cross <- .make.eval.train.dexes(eval.method = 'leave-one-out', amt.points = sdate_dim,
                                  amt.points_cor = NULL, k = 1, tail.out = FALSE)
  dates_first_year_au <- dates_uas[format(dates_uas, "%Y") == config$sdates[2]]
  years_len <- length(config$sdates)-1
  
  # Corrección porque la turbina está a 100m
  mask_ori <- startR::Start(dat = paste0(path_mask,'$var$.nc'), var = 'sftof',
                            lon = values(list(config$regions[[region]]$lon_min,config$regions[[region]]$lon_max)),
                            lat = values(list(config$regions[[region]]$lat_min,config$regions[[region]]$lat_max)),
                            synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                            return_vars = list(lat = NULL, lon = NULL), lon_var = 'lon', lat_var = 'lat',
                            lat_reorder = Sort(decreasing = FALSE), lon_reorder = CircularSort(-180,180),
                            num_procs = 1, retrieve = TRUE)
  lon_mask <- attr(mask_ori,'Variables')$common[['lon']]
  lat_mask <- attr(mask_ori,'Variables')$common[['lat']]
  lon_mask <- as.vector(lon_mask)
  lat_mask <- as.vector(lat_mask)
  mask <- drop(mask_ori)
  mask <- s2dv::CDORemap(data_array = mask[,], lons = lon_mask, lats = lat_mask, grid = grid, method = config$interpolation_method, 
                         crop = c(config$regions[[region]]$lon_min, config$regions[[region]]$lon_max, config$regions[[region]]$lat_min, config$regions[[region]]$lat_max), force_remap = TRUE)
  mask <- mask$data_array
  mask_extended <- array(mask, dim = c(dim(mask), dim(sfc_wind_obs)[1], dim(sfc_wind_obs)[4]))
  multiplication_factor <- ifelse(mask_extended == 0, 1.39, 1.29)
  multiplication_factor <- Reorder(data = multiplication_factor, order = c('sdate','lon','lat','time'))
  
  print('Calibration')
  t <- loocv
  print(t)
  wind_obs_tr <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdates', indices = cross[[t]]$train.dexes, drop = 'selected')
  wind_obs_ev <- ClimProjDiags::Subset(x = sfc_wind_obs, along = 'sdates', indices = cross[[t]]$eval.dexes, drop = FALSE)
  dcpp_tr <- ClimProjDiags::Subset(x = dcpp, along = 'sdates', indices = cross[[t]]$train.dexes, drop = 'selected')
  dcpp_ev <- ClimProjDiags::Subset(x = dcpp, along = 'sdates', indices = cross[[t]]$eval.dexes, drop = FALSE)
  obsL_tr <- ClimProjDiags::Subset(x = obsL, along = 'sdates', indices = cross[[t]]$train.dexes, drop = 'selected')

  wind_obs_tr <- wind_obs_tr * multiplication_factor[1:years_len,,,]
  multi_1 <- InsertDim(data = multiplication_factor[1,,,], posdim = 1, lendim = 1, name = 'sdate')
  wind_obs_ev <- wind_obs_ev * multi_1
  obsL_tr <- Reorder(data = obsL_tr, order = c('sdates','lon','lat','time'))
  names(dim(dcpp_ev))[1] <- 'sdates'
  names(dim(wind_obs_tr))[1] <- 'sdates'
  downscale <- Analogs(exp = dcpp_ev,
                       obs = wind_obs_tr,
                       obsL = obsL_tr,
                       obs_lats = obs_lats,
                       obs_lons = obs_lons,
                       obsL_lats = obsL_lats,
                       obsL_lons = obsL_lons,
                       exp_lats = lat_dcpp,
                       exp_lons = lon_dcpp,
                       sdate_dim = "sdates",
                       time_dim = "time",
                       region = NULL,
                       ncores = ncores,
                       lat_dim = "lat",
                       lon_dim = "lon",
                       grid_exp = config$grid_up,
                       nanalogs = 1,
                       fun_analog = 'mean',
                       loocv_window = FALSE,
                       metric = 'cor')
  
  print('FINISHED :D')
  saveRDS(object = list(wind_dcpp = downscale$data,
                        wind_obs = downscale$obs,
                        lon = downscale$lon,
                        lat = downscale$lat,
                        config = config,
                        dates = dates_uas),
          file = paste0(config$downscaling_path,'analogs_',model,'_',fyears_string,'_',member,'_',region,'_hour_',hour_to_dow,'.rds'))
}
